package gauntlet;

import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Scanner;
import java.util.regex.Pattern;
import java.util.regex.MatchResult;
import java.util.NoSuchElementException;

/*
 *TO DO:
 * * starting point(s) must be exposed
 * * clean up code
 * * throw different exceptions and recover when possible (add fatal bool to error_print()
 * * make sure cursor is inside grid
 * * make sure cursor isn't on a null block
 */

public class GauntletScanner
{

    //This var toggles any messages the user shouldn't see.
    //The user *should* see errors during level parsing.
    private static final int DEBUGGING = 0;
    private static final boolean OPTIMIZE_GRID = true;

    private static String curr_line;
    private static int curr_line_num;
    //Any coordinates added to this list need to have atributes set.
    private static ArrayList<Coords> NeedsAttrList;

    //Any coordinates in this list may not have any more blocks.
    //This is so we can guarantee that the start position won't be covered.
    private static ArrayList<Coords> ExposedBlocks;

    private static Scanner s;

    public static void main(String args[])
    {
        System.err.println("Running GauntletScanner in standalone mode...");
        if (args.length != 1) {
            System.err.println("usage: java GauntletScanner filename.lvl");
            System.exit(1);
        }
        Grid g = null;
        try {
            System.err.println("parsing level file "+args[0]+" ....");
            g = parseFile(args[0]);
            System.err.println("unparsing grid...");
            unparseGrid(g,"OUTPUT.lvl");
            System.err.println("reading levels.dat...");
            ArrayList<String> levels = getLevelIndex("levels/levels.dat");
            Iterator<String> is = levels.iterator();
            while (is.hasNext()){
                debug_print(is.next());
            }
        } catch (Exception e) {
            System.err.println("exception caught by main:"+e);
            e.printStackTrace();
        }
        System.err.println("all done");
    }

    public static Grid parseFile(String filename) throws Exception
    {
        ExposedBlocks = new ArrayList<Coords>();
        NeedsAttrList = new ArrayList<Coords>();
        MatchResult mr = null;
        //This is true if nothing that happens onscreen effects anything
        //offscreen, meaning that the Grid will only need to call update()
        //for stuff onscreen.
        //THIS IS ONLY HERE IN CASE IT BECOMES USEFUL LATER.  IT DOESN'T DO
        //ANYTHING RIGHT NOW.
        boolean staysOnscreen = true;

        //Java doesn't think that this will always be initialized.
        Grid g = null;

        s = new Scanner(new File(filename));

        //how many more blocks we expect
        int expected_blocks = 0;

        //size of playing grid
        int x_size = 0;
        int y_size = 0;
        int z_size = 0;

        //current coords
        int x = 0;
        int y = 0;
        int z = 0;

        int startCount = 0;
        int goalCount = 0;

        boolean comments_done = false;
        String curr_line = "";
        
        //strip out comments, continue when you reach the first non-comment line
        while (!comments_done) {
            curr_line = s.nextLine();
            curr_line = curr_line.replaceAll("#.*","");
            comments_done = Pattern.matches("\\d+,\\d+,\\d+:\\d+,\\d+:",curr_line);
        }

        //Scanner is a Good Thing (tm), but it's not designed to parse much 
        //more than numbers out of text and to split a Readable up by lines.
        //I use Scanner to split up the file into lines, String.replaceAll to 
        //strip out comments.  From there, I use Scanner to split up a string 
        //with a regex (ie the first line and any attribute lines) and manually
        //do the other stuff.
        
        //get x,y,z (store expected number of blocks)
        Scanner curr_scanner = new Scanner(curr_line);
        curr_scanner.findInLine("(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*:\\s*(\\d+)\\s*,\\s*(\\d+)\\s*:");
        try {
            //don't ask me why finding and matching a pattern are separate steps
            mr = curr_scanner.match();
        } catch(IllegalStateException e) {
            throw new MalformedCoordinateException(s.nextLine());
        }

        x_size = Integer.parseInt(mr.group(1));
        y_size = Integer.parseInt(mr.group(2));
        z_size = Integer.parseInt(mr.group(3));
        expected_blocks = x_size * y_size * z_size;

        //make sure that a ball rolling up a ramp at the top level 
        //won't go off the grid
        g = new Grid(x_size,y_size,z_size+1);
        
        //get cursor x,y (range check)
        g.cursorX = Integer.parseInt(mr.group(4));
        g.cursorY = Integer.parseInt(mr.group(5));
        
        if (g.cursorX >= x_size || g.cursorX < 0 ||
            g.cursorY >= y_size || g.cursorY < 0 )
            throw new CursorOutOfRangeException();

        debug_print("cursor x,y = "+g.cursorX+","+g.cursorY);
        debug_print("curr x,y,z = "+x+","+y+","+z);
        
        while( expected_blocks > 0 ) {

            curr_line = s.nextLine();
            
            //strip out comments
            if (curr_line.indexOf("#") > -1) {
                curr_line = curr_line.substring(0,curr_line.indexOf("#"));
            }
            //strip out digits and whitespace
            curr_line = curr_line.replaceAll("[\\s\\d]","");

            //go through the rest of curr_line char-by-char and add each to the
            //grid with the correct coordinates
            int i = 0;
            while (i < curr_line.length()) {
                char type = curr_line.charAt(i);
                i++;
                expected_blocks--;
            
                debug_print("found block type: "+type);

                //Teleporters mean that we'll need to update stuff offscreen
                //so set staysOnScreen to false
                if ("t".indexOf(type) != -1)
                    staysOnscreen = false;
    
                if (type == 'g') {
                    goalCount++;
                } else if (type == 's') {
                    startCount++;
                }
    
                addGridItem(g,x,y,z,type);
                
                x++;
                if (x == x_size) {
                    x = 0;
                    y++;
                
                    if (y == y_size) {
                        y = 0;
                        z++;
                    }
                }
                if (z > z_size) {
                    throw new TooManyBlocksException();
                }
            }
        }

        //add the top empty level
        /*x = 0;
        y = 0;
        z = z_size;
        while (z < z_size+1) { 
            addGridItem(g,x,y,z,'x');
            
            x++;
            if (x == x_size) {
                x = 0;
                y++;
            
                if (y == y_size) {
                    y = 0;
                    z++;
                }
            }
        }*/

        //make sure the game has at least one goal and starting point
        if (goalCount == 0) {
            throw new GoalNotFoundException();
        } else if (startCount == 0) {
            throw new StartNotFoundException();
        }
        
        //now parse through the attributes
        while( s.hasNextLine() ){
            
            curr_line = s.nextLine();
            //strip out any comments
            curr_line = curr_line.replaceAll("#.*","");

            Scanner sc = new Scanner(curr_line);
            if (sc.findInLine("(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*:\\s*\"([^\"]*)\".*") != null) {
                try {
                    mr = sc.match();
                } catch (Exception e) {
                    break;
                }
                try {
                    x = Integer.parseInt(mr.group(1));
                    y = Integer.parseInt(mr.group(2));
                    z = Integer.parseInt(mr.group(3));

                    //range check coords
                    if (x >= x_size) {
                        throw new GridIndexOutOfBoundsException("x: ("+x+","+y+","+z+")");
                    } else if (y >= y_size) {
                        throw new GridIndexOutOfBoundsException("y: ("+x+","+y+","+z+")");
                    } else if (z >= z_size) {
                        throw new GridIndexOutOfBoundsException("z: ("+x+","+y+","+z+")");
                    }
                    debug_print("x,y,z,attr => "+x+","+y+","+z+","+mr.group(4));

                    //check for null block
                    Item it = g.get(x,y,z);
                    if (it == null) {
                        throw new AttributeAssignedToNullException("null block: ("+x+","+y+","+z+")");
                    }

                    //set attributes
                    it.setAttributes(mr.group(4));
                    //remove coords from list of coords that need attributes 
                    //assigned, if appropriate
                    int i = FindIndexOf(new Coords(x,y,z), NeedsAttrList);
                    if ( i > -1 ) 
                        NeedsAttrList.remove(i);

                } catch (Exception e) {
                    System.err.println("exception caught: "+e);
                }
            } else if (sc.findInLine("GLOBAL:\"([a-zA-Z_]+)=([a-zA-Z0-9_]+)\"") != null) {
                try {
                    mr = sc.match();
                } catch (Exception e) {
                    break;
                }
                String attr_name = mr.group(1);
                String attr_val  = mr.group(2);
                debug_print("found a global attr: "+attr_name+"="+attr_val);

                //This flag says whether to ignore on stuff offscreen.
                //Setting it to true makes simple levels faster but will screw
                //up anything where movement happens offscreen.
                if (attr_name.equals("offscreen_updates") && attr_val.contains("t")) {
                    g.offscreenUpdates = true;
                    debug_print("turning off simplified rendering");
                } else if (attr_name.equals("offscreen_updates") && attr_val.contains("f")) {
                    g.offscreenUpdates = false;
                    debug_print("turning on simplified rendering");
                
                //whether or not we want all teleports to be animated
                } else if (attr_name.equals("animate_teleports")  && attr_val.contains("t")) {
                    //set global teleport anim to true
                    g.animateTeleports = true;
                    debug_print("turning on teleport animation");
                } else if (attr_name.equals("animate_teleports")  && attr_val.contains("f")) {
                    //set global teleport anim to false
                    g.animateTeleports = false;
                    debug_print("turning off teleport animation");
                }
            }
        }
        
        if (NeedsAttrList.size() != 0) {
            Coords c = null; 
            for(int i = 0; i < NeedsAttrList.size(); i++) {
                c = NeedsAttrList.get(i);
                debug_print("the block at ("+c.x+","+c.y+","+c.z+") needs an attribute set");
            }
            //I realize that I'm only throwing an exception for the last block.
            //I'm still spitting out info for all of them.
            throw new UnspecifiedAttributeException("coords: ("+c.x+","+c.y+","+c.z+")");
        }

        if (g.getTop(g.cursorX,g.cursorY) == null) {
            throw new CursorOnNullLocationException("("+g.cursorX+","+g.cursorY+")");
        }
        
        //turn any blocks that the player won't be able to see
        //into null blocks so they won't be rendered
        if (OPTIMIZE_GRID) {
            //start at 0,0,0, incrementing x,y,z in that order
            //only iterate to size-1 since blocks at the south, 
            //east and top sides will always be rendered
            debug_print("optimizing grid...");
            for (z = 0; z < z_size-1; z++) {
                for (y = 0; y < y_size-1; y++) {
                    for (x = 0; x < x_size-1; x++) {
                        //Check each block's x+1, y+1 and z+1, 
                        //only changing if all covering blocks are 
                        //non-null and immobile, since moving a movable
                        //block out of the way could expose a hole.
                        if (g.get(x,y,z)   != null &&
                            g.get(x+1,y,z) != null &&
                            g.get(x+1,y,z).mobile == false &&
                            g.get(x,y+1,z) != null &&
                            g.get(x,y+1,z).mobile == false &&
                            //ramps have holes in the sprites, so make sure we
                            //don't expose those holes by optimizing away the
                            //wrong block
                            //is the block type contained in the string "un()" ?
                            "un()".indexOf(g.get(x,y+1,z).toString()) == -1 &&
                            g.get(x,y,z+1) != null && 
                            g.get(x,y,z+1).mobile == false) 
                        {
                            g.set(x,y,z,null);
                        }
                    }
                }
            }
        }
            
        
        return g;
    }

    public static void addGridItem(Grid g, int x, int y, int z, char type) throws Exception
    {
        //For each block that needs an attribute to be meaningful, add its
        //coordinates to NeedsAttrList.  The parser will spit out an error for
        //each coordinate in the list that doesn't have an attribute.
        //Most blocks can use attributes but don't need them.

        if (type != '.' && type != 'x') {

            //make sure that we won't cover non-coverable blocks, ie ones
            //with Coords in the ExposedBlocks array, except with starting points
            int i = FindIndexOfXY(new Coords(x,y,0), ExposedBlocks);
            if ( i > -1 && type != 's') {
                debug_print("block at "+x+","+y+","+z+" covers a non-coverable block "+ExposedBlocks.get(i));
                throw new BlockCoversNonCoverableException(type+"("+x+","+y+","+z+") covers "+ExposedBlocks.get(i));
            }
        }

        Item i = null;

        switch (type) {
            //goal
            case 'g':
                debug_print("block at "+x+","+y+","+z+" is a goal");
                i = new Goal(g,x,y,z);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            //immovable block
            case 'i':
                debug_print("block at "+x+","+y+","+z+" is immovable");
                i = new Block(g,x,y,z,false);
                break;
            //mobile block
            case 'm':
                debug_print("block at "+x+","+y+","+z+" is movable");
                i = new Block(g,x,y,z,true);
                break;
            //turbo block
            case 'A':
                debug_print("block at "+x+","+y+","+z+" is turbo (N)");
                i = new TurboBlock(g,x,y,z,Direction.NORTH);
                break;
            case 'V':
                debug_print("block at "+x+","+y+","+z+" is turbo (S)");
                i = new TurboBlock(g,x,y,z,Direction.SOUTH);
                break;
            case '>':
                debug_print("block at "+x+","+y+","+z+" is turbo (E)");
                i = new TurboBlock(g,x,y,z,Direction.EAST);
                break;
            case '<':
                debug_print("block at "+x+","+y+","+z+" is turbo (W)");
                i = new TurboBlock(g,x,y,z,Direction.WEST);
                break;
            //ice
            case 'c':
                debug_print("block at "+x+","+y+","+z+" is ice");
                i = new Ice(g,x,y,z);
                break;
            //nothing (don't put any block at these coords)
            case '.':
            case 'x':
                debug_print("block at "+x+","+y+","+z+" is empty");
                break;
            //starting point
            case 's':
                debug_print("block at "+x+","+y+","+z+" is a starting point");
                i = new Ball(g,x,y,z);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            //teleporter
            case 't':
                debug_print("block at "+x+","+y+","+z+" is a transporter");
                i = new Teleport(g,x,y,z);
                NeedsAttrList.add(new Coords(x,y,z,type));
                break;
            //ramps
            case 'n':
                debug_print("block at "+x+","+y+","+z+" is a ramp (up)");
                i = new Ramp(g,x,y,z,Direction.NORTH);
                break;
            case 'u':
                debug_print("block at "+x+","+y+","+z+" is a ramp (down)");
                i = new Ramp(g,x,y,z,Direction.SOUTH);
                break;
            case '(':
                debug_print("block at "+x+","+y+","+z+" is a ramp (left)");
                i = new Ramp(g,x,y,z,Direction.WEST);
                break;
            case ')':
                debug_print("block at "+x+","+y+","+z+" is a ramp (right)");
                i = new Ramp(g,x,y,z,Direction.EAST);
                break;
            //halfpipes
            case 'b':
                debug_print("block at "+x+","+y+","+z+" is a halfpipe (NE)"); 
                i = new Pipe(g,x,y,z,Direction.NORTH,Direction.EAST);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            case 'p':
                debug_print("block at "+x+","+y+","+z+" is a halfpipe (SE)");
                i = new Pipe(g,x,y,z,Direction.SOUTH,Direction.EAST);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            case 'q':
                debug_print("block at "+x+","+y+","+z+" is a halfpipe (SW)");
                i = new Pipe(g,x,y,z,Direction.SOUTH,Direction.WEST);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            case 'd':
                debug_print("block at "+x+","+y+","+z+" is a halfpipe (NW)");
                i = new Pipe(g,x,y,z,Direction.NORTH,Direction.WEST);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            case '|':
                debug_print("block at "+x+","+y+","+z+" is a halfpipe (NS)");
                i = new Pipe(g,x,y,z,Direction.NORTH,Direction.SOUTH);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;
            case '-':
                debug_print("block at "+x+","+y+","+z+" is a halfpipe (EW)");
                i = new Pipe(g,x,y,z,Direction.EAST,Direction.WEST);
                ExposedBlocks.add(new Coords(x,y,z,type));
                break;

            default:
                throw new InvalidBlockTypeException(type+"");
        }
        //add item to grid
        if(i!=null) g.set(x,y,z,i);
        return;
    }

    public static boolean unparseGrid(Grid g, String filename )
    {
        int csr_x = g.cursorX;
        int csr_y = g.cursorY;

        String gStr = "";

        gStr = g.width() + "," + g.length() + "," + g.height() + ":";
        gStr = gStr + csr_x + "," + csr_y + ":";

        int x = 0;
        int y = 0;
        int z = 0;
        int max_x = g.width();
        int max_y = g.length();
        int max_z = g.height();
        Item i;

        while( z < max_z ) {
            z++;
            gStr = gStr + "\n";
            y = 0;

            while( y < max_y ) {
                y++;
                gStr = gStr + "\n";
                x = 0;

                while (x < max_x ) {
                    x++;
                    i = g.get(x-1,y-1,z-1);
                    if (i == null) {
                        gStr = gStr + ".";
                    } else {
                        gStr = gStr + i;
                    }
                }
            }
        }

        Iterator<Item> ii = g.iterator();
        while ( ii.hasNext() ) {
            i = ii.next();
            if (i.hasAttributes) {
                gStr = gStr + i.x + "," + i.y + "," + i.z + ":";
                gStr = gStr + "\"" + i.getAttributeString() + "\"\n";
            }
        }

        try{
            FileWriter fw = new FileWriter(filename);
            fw.write(gStr);
            fw.close();
        } catch (IOException e) {
            System.err.println("caught IOException: "+e);
            return false;
        }
        return true;
    }


    public static void debug_print(String s)
    {
        if (DEBUGGING != 0)
            System.err.println(s);
    }

    //if a block with matching x,y,z coords is in l, return its index
    private static int FindIndexOf(Coords c, ArrayList<Coords> l)
    {
        int i = 0;
        int j = l.size();
        Coords currCoords = (Coords)null;
        while(i < j){
            currCoords = l.get(i);
            if (c.equals(currCoords))
                return i;
            i++;
        }
        return -1;
    }

    public static ArrayList<String> getLevelIndex(String filename)
    {
        ArrayList<String> levels = new ArrayList<String>();
        try {
            FileReader fr = new FileReader(filename);
            Scanner s = new Scanner(fr);
			if(s.hasNext())
                levels.add(s.next() + ".lvl");
			if(s.hasNext())
				s.next();
			
        } catch(Exception e) {
            System.err.println("caught Exception:"+e);
            System.exit(1);
        }
        return levels;
    }


    //if a block with matching x and y coords is in l, return its index
    private static int FindIndexOfXY(Coords c, ArrayList<Coords> l)
    {
        int i = 0;
        int j = l.size();
        Coords currCoords = (Coords)null;
        while(i < j){
            currCoords = l.get(i);
            if (c.equalsXY(currCoords))
                return i;
            i++;
        }
        return -1;
    }

    private static class Coords
    {
        public final int x,y,z;
        public final char type;

        public Coords(int _x, int _y, int _z, char _type)
        {
            x = _x;
            y = _y;
            z = _z;
            type = _type;
        }

        public Coords(int _x, int _y, int _z)
        {
            x = _x;
            y = _y;
            z = _z;
            type = '?';
        }

        public String toString()
        {
            return type+"("+x+","+y+","+z+")";
        }

        public boolean equals(Coords c)
        {
            if (c == null)
                return false;
            return (this.x == c.x && this.y == c.y && this.z == c.z);

        }

        //only compare x and y coords
        public boolean equalsXY(Coords c)
        {
            
            debug_print("comparing "+c+" and "+this);
            if (c == null)
                return false;
            return (this.x == c.x && this.y == c.y);

        }
    }
    
    static class CursorOnNullLocationException extends Exception
    {
        CursorOnNullLocationException(){super();}
        CursorOnNullLocationException(String s){super(s);}
    }
    
    static class AttributeAssignedToNullException extends Exception
    {
        AttributeAssignedToNullException(){super();}
        AttributeAssignedToNullException(String s){super(s);}
    }
    
    static class GridIndexOutOfBoundsException extends Exception
    {
        GridIndexOutOfBoundsException(){super();}
        GridIndexOutOfBoundsException(String s){super(s);}
    }
    
    static class UnspecifiedAttributeException extends Exception
    {
        UnspecifiedAttributeException(){super();}
        UnspecifiedAttributeException(String s){super(s);}
    }
    
    static class CursorOutOfRangeException extends Exception
    {
        CursorOutOfRangeException(){super();}
        CursorOutOfRangeException(String s){super(s);}
    }
    
    static class StartNotFoundException extends Exception
    {
        StartNotFoundException(){super();}
        StartNotFoundException(String s){super(s);}
    }

    static class GoalNotFoundException extends Exception
    {
        GoalNotFoundException(){super();}
        GoalNotFoundException(String s){super(s);}
    }

    static class TooFewBlocksException extends Exception
    {
        TooFewBlocksException(){super();}
        TooFewBlocksException(String s){super(s);}
    }

    static class InvalidBlockTypeException extends Exception
    {
        InvalidBlockTypeException(){super();}
        InvalidBlockTypeException(String s){super(s);}
    }

    static class MalformedCoordinateException extends Exception
    {
        MalformedCoordinateException(){super();}
        MalformedCoordinateException(String s){super(s);}
    }
    
    static class BlockCoversNonCoverableException extends Exception
    {
        BlockCoversNonCoverableException(){super();}
        BlockCoversNonCoverableException(String s){super(s);}
    }
    
    static class TooManyBlocksException extends Exception
    {
        TooManyBlocksException(){super();}
        TooManyBlocksException(String s){super(s);}
    }
}
