package gauntlet;

// JFC
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.Scanner;
import java.io.FileWriter;

// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.engine.BaseGraphics;
import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.audio.MidiRenderer;
import com.golden.gamedev.util.FileUtil;
import com.golden.gamedev.util.Utility;
import com.golden.gamedev.funbox.*;

import gauntlet.menu.*;


/**
 * The Game Frame Work.
 *
 * Initializes the game object (menu screen, and game mode).
 *
 * Consists of :
 * - Puzzle class           -> the game mode
 * - Text class                 -> text score
 *
 * - menu package
 *   - Menu class           -> the base game object of all menus
 *   - Title class      -> title screen
 *   - Control class    -> control option
 *   - Story class      -> story screen
 *   - Option class     -> game option
 */
public class Gumball extends GameEngine
{
 /************************** GAME STATE CONSTS *******************************/

    public static final int TITLE_SCREEN = 0,   // main menu
                            GAME_MODE = 1,      // play-the-game!!
                            GAME_EXIT = 2;


 /**************************** GAME SETTINGS *********************************/

    public boolean  showFPS = false; //for debugging


 /**************************** ROBO CONTROLLER *******************************/

    public int[]    controller;

    public LevelScanner theLevelInfo;

    public int levelIndex;

    public int levelsCompleted;
	public int levelContinue;

 /****************************************************************************/
 /*************************** INIT GAME RESOURCES ****************************/
 /****************************************************************************/

    public void initResources()
    {
        // load controller--determines which keys to use for input
        loadController();
        loadLevelInfo();
        getPlayer();
        //levelIndex = 0;
        setFPS(100);
    }

    //
    // loadController():
    //
    // read key information from file--determines which keys to use
    // also has default key settings if file not found
    //
    private void loadController()
    {
        try
        {
            //loading of config info?
            File input = new File("input.dat");

            if (input.exists() == false) {
                throw new FileNotFoundException();
            }

            String[] lines = FileUtil.fileRead(input);

            /*
            controller = new int[] {
                Integer.parseInt(lines[1]), Integer.parseInt(lines[3]),
                Integer.parseInt(lines[5]), Integer.parseInt(lines[7]),
                Integer.parseInt(lines[9]), Integer.parseInt(lines[11]),
                Integer.parseInt(lines[13])
            };
            */

            controller = new int[]
                {
                    Integer.parseInt(lines[1]), Integer.parseInt(lines[3]),
                    Integer.parseInt(lines[5]), Integer.parseInt(lines[7]),
                    Integer.parseInt(lines[9]), Integer.parseInt(lines[11])
                };

        } catch (Exception e) {
            // can't open/read controller file
            // set default robo controller
            controller = new int[] {
                KeyEvent.VK_LEFT, KeyEvent.VK_RIGHT,        // left-right
                KeyEvent.VK_UP, KeyEvent.VK_DOWN,               // up-down
                KeyEvent.VK_SPACE, KeyEvent.VK_R,     // fly-fire
                KeyEvent.VK_SHIFT                                               // shift
            };
        }

    }//end loadController()

    private void loadLevelInfo()
    {
        try
        {
            theLevelInfo = new LevelScanner("levels/levels.dat");
            //System.out.println(theLevelInfo.getLevelCount());
        }
        catch( Exception e)
        {
            System.out.println(e);
        }
    }


 /****************************************************************************/
 /**************************** GET THE GAME!! ********************************/
 /****************************************************************************/

    public GameObject getGame(int GameID)
    {
        switch (GameID) {
            case TITLE_SCREEN:      return new Title(this);
            case GAME_MODE:         return new Puzzle(this);
            case GAME_EXIT:         saveGame();
                                    return null;
        }

        return null;
    }


 /****************************************************************************/
 /******************** UPDATE AND RENDER COMMON GAME *************************/
 /****************************************************************************/

    public void update(long elapsedTime) { }

    public void render(Graphics2D g) {
        if (showFPS) {
            drawFPS(g, 20, 20);
        }
    }

 /****************************************************************************/
 /***************************** MAIN-CLASS ***********************************/
 /****************************************************************************/

    public static void main(String[] args)
    {
        //load the game
        GameLoader game = new GameLoader();

        //setup the game
        game.setup(new Gumball()
        {
            protected void initEngine()
            {
                super.initEngine();

                //set sound and music based on preferences in dialog
                //bsSound.setActive(sound.isSelected());
                //bsMusic.setActive(sound.isSelected());
                bsSound.setActive(false);
                bsMusic.setActive(false);
            }
        },  new Dimension(640, 480), false); /*false => not fullscreen*/

        //let's go
        game.start();

    }//end main()

    { distribute = true; }

    public void saveGame()
    {
        theLevelInfo.saveScores();
        savePlayer();
        System.exit(0);
    }

    protected void notifyError(Throwable e)
    {
        new ErrorNotificationDialog(e, bsGraphics,
                                   "Gumball Gauntlet 0.1", "michelhoo@yahoo.com");
    }

    public void savePlayer()
    {
        FileWriter writeFile;
        levelsCompleted = Math.max(levelIndex, levelsCompleted);
        try
        {
            writeFile = new FileWriter("player.dat");
            //theScores = levelScoreLists.get(i);
            //scoreIterator = theScores.iterator();
            //System.out.println(scoreFiles.get(i));
            writeFile.write( levelsCompleted+ "\n");
			writeFile.write(levelContinue + "\n");
            writeFile.close();


        }
        catch(Exception e)
        {
            System.out.println(e);
        }
    }

    public void getPlayer()
    {
        try
        {
            //System.out.println("levels/"+ scoreFiles.get(levelIndex));
            FileReader fr = new FileReader("player.dat");

            Scanner s = new Scanner(fr);
            //Need Checks here
            levelsCompleted = s.nextInt();
			levelContinue = s.nextInt();

            if(levelsCompleted < theLevelInfo.getLevelCount())
                levelIndex = levelsCompleted;
            else
            {
                levelIndex = 0;
                levelsCompleted = 0;
            }
            //System.out.println(levelIndex);
        }
        catch(FileNotFoundException e)
        {
            levelsCompleted = 0;
			levelContinue = 0;
            levelIndex = 0;
        }
        catch(Exception e)
        {
            levelsCompleted = 0;
			levelContinue = 0;
            levelIndex = 0;
            //System.err.println("caught Exception, why me?:"+e);
            //System.exit(1);
        }
    }

		protected void notifyExit()
		{
			saveGame();
		}


}//end class Gumball

