package gauntlet;

import com.golden.gamedev.*;
import com.golden.gamedev.object.GameFont;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import com.golden.gamedev.object.GameFontManager;
import java.awt.image.BufferedImage;
import com.golden.gamedev.util.ImageUtil;
import java.awt.event.*;

public class InGameUI extends GameObject 
{
	//fonts
	private GameFont small;
	private GameFont font;
	private GameFontManager fontManager;
	private int width;
	private int height;
	private BufferedImage backgroundImage;
	private LevelScanner internalLevelInfo;
	private int selectedIndex;
	private int totalOptions;
	private int winState;
	private boolean inMenu;
	private boolean win;
	private boolean fail;
	private boolean gameWon;
	private boolean newBestTime;
	private boolean emptyName;
	private boolean skipPrompt;
	private int level;
	private String levelName;
	private String bestPlayerName;
	private String bestPlayerTime;
	private String levelCode;
	private String playerName;

	public InGameUI(GameFontManager gameFontManage, int newWidth, int newHeight, String backFileName,
									GameEngine parent)
	{
		super(parent); //note: "parent" now is a *global* for us b/c of class GameObject
		fontManager = gameFontManage;
		small = fontManager.getFont(new Font("Arial", Font.BOLD, 16));
		font = fontManager.getFont(new Font("Verdana", Font.BOLD, 16));
		width = newWidth;
		height = newHeight;
		backgroundImage = getImage(backFileName);
		internalLevelInfo = ((Gumball) parent).theLevelInfo;
		selectedIndex = 0;
		totalOptions = 3;
		winState = CHECK_BEST_TIME;
		inMenu = false;
		win = false;
		fail = false;
		gameWon = false;
		newBestTime = false;
		emptyName = false;
		skipPrompt = false;
		level = ((Gumball) parent).levelIndex;
		levelName = formatString(internalLevelInfo.getPrettyLevelName(level), 110);
		bestPlayerName = formatString(internalLevelInfo.getStringName(level), 110);
		bestPlayerTime = formatString(internalLevelInfo.getStringScore(level), 110);
		levelCode = formatString(internalLevelInfo.getCode(level), 110);
		playerName = new String();

	}

	public void refreshGUI()
	{
		level = ((Gumball) parent).levelIndex;
		levelName = formatString(internalLevelInfo.getPrettyLevelName(level), 110);
		bestPlayerName = formatString(internalLevelInfo.getStringName(level), 110);
		bestPlayerTime = formatString(internalLevelInfo.getStringScore(level), 110);
		levelCode = formatString(internalLevelInfo.getCode(level), 110);
		playerName = new String();
		inMenu = false;
		win = false;
		fail = false;
		gameWon = false;
		newBestTime = false;
		emptyName = false;
		skipPrompt = false;
		playerName = "";		
		winState = CHECK_BEST_TIME;
	}

	public void GUIrender(Graphics2D g) 
	{
		printInGameGUI(g);

		if(inMenu)
			printInGameMenu(g);
		else if(fail)
			printFail(g);
		else if (newBestTime)
			printBestTime(g);
		else if(win)
			printWin(g);
		else if(gameWon)
			printGameWon(g);
		else if(skipPrompt)
			printSkipPrompt(g);
	}
	
	//need to implement these b/c we extend GameObject,
	//but don't touch these
	public void render(Graphics2D g){}
	public void update(long elapsedTime){}
	public void initResources(){} 

	//print a message to the screen when the user loses a game	
	public void printFail(Graphics2D g)
	{
		int fillX = 400;
		int fillY = 100;

		g.setColor(Color.BLACK);
		g.fillRect((width - fillX)/2, (height-fillY)/2, fillX, fillY);
		g.setColor(Color.RED);
		drawText(g, "You Fail!", 10, false);
		drawText(g, "Press Enter to Restart Level", 12, false);
	}

	public void drawBackground(Graphics2D g)
	{
		g.drawImage(backgroundImage, 480, 0, null);
	}

	public void printWin(Graphics2D g)
	{
		int fillX = 300;
		int fillY = 100;

		g.setColor(Color.BLACK);
		g.fillRect((width - fillX)/2, (height-fillY)/2, fillX, fillY);
		g.setColor(Color.RED);
		drawText(g, "You Win!", 10, false);
		drawText(g, "Press Enter to Load Next Level", 12, false);
	}

	public void printGameWon(Graphics2D g)
	{
		int fillX = 400;
		int fillY = 100;

		g.setColor(Color.BLACK);
		g.fillRect((width - fillX)/2, (height-fillY)/2, fillX, fillY);
		g.setColor(Color.RED);
		drawText(g, "You Beat the Game!", 10, false);
		drawText(g, "Press Enter to return to the Main Menu", 12, false);
	}

	public void printBestTime(Graphics2D g)
	{
		int fillX = 300;
		int fillY = 100;

		g.setColor(Color.BLACK);
		g.fillRect((width - fillX)/2, (height-fillY)/2, fillX, fillY);
		g.setColor(Color.RED);
		drawText(g, "You Have A Best Time!", 10, false);
		drawText(g, "Type your name: " + playerName, 12, false);
		drawText(g, "ENTER when done.", 13, false);
	}

	public void printInGameMenu(Graphics2D g)
	{
		int fillX = 300;
		int fillY = 100;

		g.setColor(Color.BLACK);
		g.fillRect((width - fillX)/2, (height-fillY)/2, fillX, fillY);
		g.setColor(Color.RED);
		drawText(g, "Restart Level", 10, (selectedIndex == 0));
		drawText(g, "Resume Game", 11, (selectedIndex == 1));
		drawText(g, "Exit to Main Menu", 12, (selectedIndex == 2));
		drawText(g, "Exit Game", 13, (selectedIndex == 3));
	}

	public void menu(long elapsedTime)
	{
		inMenu = true;

		if (keyPressed(KeyEvent.VK_UP))
		{
			selectedIndex--;
			if (selectedIndex < 0) selectedIndex = totalOptions;
		}
		else if (keyPressed(KeyEvent.VK_DOWN))
		{
			selectedIndex++;
			if (selectedIndex > totalOptions) selectedIndex = 0;
		}
		else if (keyPressed(KeyEvent.VK_SPACE) || keyPressed(KeyEvent.VK_ENTER))
		{
			switch(selectedIndex)
			{
				case 0:
					//user indicated to restart level; that's fine. The state
					//DEFAULT will catch the moment when MAX_RESETS has been
					//exceeded. We just need to restart the level and return
					//to that state.
					Puzzle.numResets++;
					Puzzle.updateState = Puzzle.START_LEVEL;
					inMenu = false;
					break;
				case 1:
					Puzzle.updateState = Puzzle.DEFAULT;
					inMenu = false;
					break;
				case 2:
					parent.nextGameID = Gumball.TITLE_SCREEN;
					Puzzle.updateState = Puzzle.GAME_CLOSE;
					finish();
					break;
				case 3: //exit game
					parent.nextGameID = Gumball.GAME_EXIT;
					Puzzle.updateState = Puzzle.GAME_CLOSE;
					finish();
					break;
			}
		}
	}

	final int CHECK_BEST_TIME = 0,
		NEW_WIN = 1,
		NEW_BEAT_GAME = 2;
		
	

	public void winPopup(long elapsedTime)
	{
		switch(winState)
		{
			case 0: 
				checkBestTime();				
				break;
			case 1:
				newWin();
				break;
			case 2:
				newBeatGame();
				break;
		}
	}

	public void newWin()
	{
		if(((Gumball) parent).levelIndex >= internalLevelInfo.getSize() - 1)
			winState = NEW_BEAT_GAME;
	
		if(winState == NEW_WIN)
			win = true;
		
		if(keyPressed(KeyEvent.VK_ENTER))
		{
			((Gumball) parent).levelIndex++;
			if(((Gumball) parent).levelsCompleted <((Gumball) parent).levelIndex)
                ((Gumball) parent).levelsCompleted = ((Gumball) parent).levelIndex;
			((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;

			//the user won, so we go directly to start level; no need
			//to prompt whether user wants to skip the level because the user beat the level. 
			Puzzle.numResets = 0;  //InGameUI is repsonsible for resetting this!!
			Puzzle.updateState= Puzzle.START_LEVEL;
			win = false;
		}
	}

	public void checkBestTime()
	{
		if(Puzzle.currentScore < internalLevelInfo.getLongScore(((Gumball) parent).levelIndex))
		{
			newBestTime = true;
			// read key input
			//System.out.println("Yes");
			int input = bsInput.getKeyPressed();

			if(keyPressed(KeyEvent.VK_ENTER))
			{
				if(playerName.length() == 0)
				{
					playerName = "Perezoso";
					internalLevelInfo.newScore(playerName, Puzzle.currentScore, level);
					newBestTime = false;
					winState = NEW_WIN;
				}
				else
				{
					internalLevelInfo.newScore(playerName, Puzzle.currentScore, level);
					newBestTime = false;
					winState = NEW_WIN;
				}
			}
			else if(keyPressed(KeyEvent.VK_BACK_SPACE))
			{
				if(playerName.length() > 0)
					playerName = playerName.substring(0,playerName.length() - 1);
			}
			else if (input != bsInput.NO_KEY && input >= 0x30 && input <= 0x5A && input != 0x3B && input != 0x3D) 
			{
						
				if (input != KeyEvent.VK_ESCAPE) 
				{
					if(playerName.length() < 10)
						playerName = playerName + KeyEvent.getKeyText(input);
				}
			}
		}
		else
		{
			winState = NEW_WIN;
		}
	}

	public void newBeatGame()
	{
		gameWon = true;
		if (keyPressed(KeyEvent.VK_ENTER))
		{
			parent.nextGameID = Gumball.TITLE_SCREEN;
			Puzzle.updateState = Puzzle.GAME_CLOSE;
			gameWon = false;
			Puzzle.numResets = 0;
			((Gumball) parent).levelIndex = 0;
		}
	}

	public void failPopup(long elapsedTime)
	{
		fail = true;

		if (keyPressed(KeyEvent.VK_ENTER))
		{
			//a failure is like a reset; in fact, below we
			//actually *force* a reset when the user fails.
			//So update numResets accordingly.
			Puzzle.numResets++;
			Puzzle.updateState = Puzzle.START_LEVEL;
			fail = false;
		}
		else if(keyPressed(KeyEvent.VK_ESCAPE))
		{
			Puzzle.updateState = Puzzle.MENU_MODE;
		}
	}


	void drawText(Graphics2D g, String text, int line, boolean selected)
	{
		if (selected)
		{
			// draw selected rectangle
			g.setColor(Color.YELLOW);

			g.fillRect((width/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}

		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.CENTER,
			0, line*20, width);
	}

	public void drawLeftText(Graphics2D g, String text, int line, int offset) 
	{
		g.setColor(Color.BLACK);

		font.drawString(g, text, GameFont.LEFT,
			offset, line*20, getWidth());
	}

	public void drawRightText(Graphics2D g, String text, int line, int offset) 
	{

		g.setColor(Color.BLACK);

		font.drawString(g, text, GameFont.RIGHT,
			offset, line*20, getWidth());
	}

	public void printInGameGUI(Graphics2D g)
	{
		g.setColor(Color.BLACK);
		drawLeftText(g, "Level " + (level + 1) + ": ",5, 520);
		drawRightText(g, levelName,	6, -20);
		drawLeftText(g, "Level Code: ", 7, 520);
		drawRightText(g, levelCode,	8, -20);
		drawLeftText(g, "Best Player:", 19, 520);
		drawRightText(g, bestPlayerName , 20, -20);
		drawLeftText(g, "Best Time:", 21, 520);
		drawRightText(g, bestPlayerTime  + "s", 22, -20);
		drawLeftText(g, "Time: " + Puzzle.currentScore, 13, 520);
	}

	public void resetsExceeded(long elapsedTime)
	{
		skipPrompt = true;
		if(((Gumball) parent).levelIndex + 1 < internalLevelInfo.getLevelCount())
		{
			if (keyPressed(KeyEvent.VK_Y))
			{
				((Gumball) parent).levelIndex++;

				if (((Gumball) parent).levelsCompleted < ((Gumball) parent).levelIndex)
					((Gumball) parent).levelsCompleted = ((Gumball) parent).levelIndex;
				((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;

				Puzzle.numResets = 0;
				skipPrompt = false;
				Puzzle.updateState= Puzzle.START_LEVEL;
			}
			else if (keyPressed(KeyEvent.VK_N))
			{
				Puzzle.numResets = 0;
				skipPrompt = false;
				Puzzle.updateState = Puzzle.START_LEVEL;
			}
		}
		else
		{
			Puzzle.numResets = 0;
			skipPrompt = false;
			Puzzle.updateState = Puzzle.START_LEVEL;
		}

	}//end resetsExceeded()

	public void printSkipPrompt(Graphics2D g)
	{
		int fillX = 300;
		int fillY = 100;

		g.setColor(Color.BLACK);
		g.fillRect((width - fillX)/2, (height-fillY)/2, fillX, fillY);
		g.setColor(Color.RED);
		drawText(g, "Do you wish to skip this level?", 10, false);
		drawText(g, "(Y)es or (N)o", 12, false);
	}

	public String formatString(String theString, int width)
	{
		while(font.getWidth(theString) > width)
		{
			theString = theString.substring(0,theString.length() -1);
		}
		return theString;
	}
}
