package gauntlet;

// JFC
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.ArrayList;

	// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.*;
import com.golden.gamedev.util.ImageUtil;
import com.golden.gamedev.funbox.ErrorNotificationDialog;
import com.golden.gamedev.engine.input.AWTInput;

/**
 * Glossary:
 * Piece    -> puzzle chip
 * Shuffle  -> scrambling the puzzle piece
 * Swap         -> moving piece with another piece
 */

public class LevelEdit extends Game
{
	//globals

	private static final int    VIEWING = 0,        // viewing full puzzle image
		PLAYING = 1,
		SOLVED = 2,
		SWAPPING = 3,       // swapping animation
		SHUFFLING = 4;      // shuffling animation

	private int gameState;
	private int level = 1;

	private String[] imageFileNames = new String[5];
	
	// full puzzle image
	private BufferedImage image;

	// full puzzle image size
	private int width, height;

	//Cursor position
	private int cursorX, cursorY;

	//Grid has been saved
	private boolean gridSave = false;

	// each puzzle piece is a sprite (see the Piece class at the bottom)
	private Piece[][] piece;

	//file name
	private static String levelName;

	//current sprite int
	private int currentSprite;

	//awt input class
	private AWTInput theInput;

	//sprite array
	private Sprite[] levelSprites;

	//draw lines
	private boolean lines = true;

	//boolean Array
	private boolean[] usedSprite = new boolean[5];

	//Holder sprite
	Sprite holderSprite;

	//SpriteGroup holding all item sprites
	SpriteGroup theGroup = new SpriteGroup("theGroup");

	//Current Level
	int currentLevel;

	// piece images
	private BufferedImage[] pieceImages;

	// default coordinate of each piece
	// defaultPos[0][0] -> 0, 0
	private Point[][] defaultPos;

	// current empty slot, indicates no piece in this position
	private Point emptyPiece = new Point();

	// store pieces for shuffling animation
	private ArrayList   slidePieces = new ArrayList();

	// store swapping pieces
	private ArrayList   swapPieces = new ArrayList();

	//fonts
	private GameFont    small;
	private GameFont    f;

	//our globals
	static Grid grid;
	Cursor cursor;


	/****************************************************************************/
	/************************** INIT GAME RESOURCES *****************************/
	/****************************************************************************/

	public void initResources()
	{
		//show the cursor = engine call
		showCursor();

		imageFileNames[0] = "images/BlueBall.png";
		imageFileNames[1] = "images/BlueSquare.png";
		imageFileNames[2] = "images/BlueHole.png";
		imageFileNames[3] = "images/YellowSquare.png";
		imageFileNames[4] = "images/RedCursor.png";
		
		cursorX = 0;
		cursorY = 0;

		for(int i = 0; i<5; i++)
		{
			usedSprite[i] = false;
		}

		// init font
		small = fontManager.getFont(new Font("Arial", Font.BOLD, 16));
		f = fontManager.getFont(new Font("Verdana", Font.BOLD, 26));

		// init new level
		initLevel();
	}

	/****************************************************************************/
	/************************* LEVEL INITIALIZATION *****************************/
	/****************************************************************************/

	private void initLevel()
	{
		holderSprite = new Sprite();
		super.initEngine();
		theInput = (AWTInput)bsInput;
		levelSprites = new Sprite[10];

		currentLevel = 0;

		// initial state -> shuffle animation
		gameState = SHUFFLING;

		//create a button that says level 1 on it
		//btnLevel = initButton("Level "+level);

		for(int i = 0; i <5; i++)
		{
			if(!usedSprite[i])
			{
				//get the image
				BufferedImage bufImage = getImage(imageFileNames[i], true);
				levelSprites[i] = new Sprite(bufImage, 40, 20 + 32 * i);
				theGroup.add(levelSprites[i]);
			}
		}

		try 
		{ //get the grid
			//grid = GauntletScanner.parseFile("L1.lvl");
		} 
		catch (Exception e) { System.err.println(e); e.printStackTrace(); }

		
		fillGrid();

		if(grid == null)
		{
			System.err.println("grid is null");
			System.exit(0);
		}

		//no longer using pieceImages; now everything
		//is in grid

		//so we need to get each filename for the graphics
		//and create a sprite for it
		/*
		for (Item item : grid)
		{
			//create full path to sprite image
			String filename = "images/" + item.imageFile;

			//get the image
			BufferedImage bufImage = getImage(filename, true);

			//allocate screen coords
			int screenX, screenY;

			//compute initial screen coords
			screenX = (26 * (item.x+1)) - (6 * item.z) + 120;
			screenY = (26 * (item.y+1)) - (6 * item.z);

			//create a sprite for this item
			item.sprite = new Piece(bufImage, screenX, screenY, item);
		}
		*/

	}//end initLevel()

	//
	// slideAnimation()
	//
	private void slideAnimation(Point from, Point to)
	{
		// we move the puzzle piece from "from point" to "to point"
		// add code here
	}

	//
	// finishSlide()
	//
	// end slide animation
	// we swap the image 'n number
	// then return both pieces back to its position
	//
	void finishSlide()
	{
		//add code here
	}


	/****************************************************************************/
	/***************************** UPDATE GAME **********************************/
	/****************************************************************************/

	public void update(long elapsedTime)
	{
		//potentially a lot of code could be added here

		//loop through items in the proper order
		for (Item item : grid)
		{
			//update it
			Piece piece = (Piece)item.sprite;
			piece.update(elapsedTime);
		}

		if(theInput.isMousePressed(1))
		{
			//System.out.println("Pressed");
			currentSprite = getNewSprite(theInput.getMouseX(), theInput.getMouseY());
			if(currentSprite != -1)
                holderSprite = levelSprites[currentSprite];
		}
		if(theInput.isMouseDown(1))
		{
			//System.out.println("Down");
			holderSprite.forceX(theInput.getMouseX() - 12);
			holderSprite.forceY(theInput.getMouseY()- 12);
		}
		if(theInput.isMouseReleased(1))
		{
			//System.out.println("Yessir");
			findGridPosition(theInput.getMouseX(), theInput.getMouseY());
			
			if(currentSprite != -1)
				usedSprite[currentSprite] = false;
			holderSprite = new Sprite();
		}

		switch (bsInput.getKeyPressed()) 
		{
				// grid
			case KeyEvent.VK_X:
				if(lines)
					lines = false;
				else
					lines = true;
				break;

				// save
			case KeyEvent.VK_S:
					saveGrid();
				break;
				// up
			case KeyEvent.VK_UP:
				if(currentLevel + 1 < grid.height())
					currentLevel++;
				break;

				// down
			case KeyEvent.VK_DOWN:
				if(currentLevel -1 >= 0)
					currentLevel--;
				break;
		}



	}//end update()


	/****************************************************************************/
	/**************************** RENDER GAME ***********************************/
	/****************************************************************************/

	public void render(Graphics2D g)
	{
		// clear background of game
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getWidth(), getHeight());

		//loop through items in the proper order
		for (Item item : grid)
		{
			//render it
			Piece piece = (Piece)item.sprite;
			piece.render(g);
			//if (cursor.item == item) cursor.render(g);
		}
	
		g.setColor(Color.black);
		f.drawString(g, "Level: " + (currentLevel + 1),40, 200);
		
		if(gridSave)
			f.drawString(g, "Grid Saved",40, 220);


		drawNewItems();
		if(lines)
			drawLines(g);

		theGroup.render(g);

	}//end render()

	public int getNewSprite(int x, int y)
	{
		for(int i = 0; i < 5; i++)
		{
			if(x > levelSprites[i].getX() && x < (levelSprites[i].getX() + 32))
			{
				if(y > levelSprites[i].getY() && y < (levelSprites[i].getY() + 32))
				{
					//System.out.println("Yes!");
					usedSprite[i] = true;
					return i;
				}
			}
		}

		return -1;
	}

	public void findGridPosition(int x, int y)
	{
		int xGridSize;
		int yGridSize;

		yGridSize = grid.length();
		xGridSize = grid.width();

		int screenX = (26  - (6 * currentLevel - 1)) + 119;
		int screenY = (26  - (6 * currentLevel - 1)) - 1;

		int holderX = 0;
		int holderY = 0;

		int gridPositionX = -1;
		int gridPositionY = -1;

		if(x >= screenX && x <= screenX + 26*xGridSize)
		{
			if(y >= screenY && y <= screenY + 26*yGridSize)
			{
				while(holderX <= xGridSize)
				{
					if(x >= screenX + 26*holderX && x < screenX + 26*(holderX + 1))
					{
						gridPositionX = holderX;
						holderX = xGridSize + 1;
					}
					holderX++;
				}

				while(holderY <= yGridSize)
				{
					if(y >= screenY + 26*holderY && y < screenY + 26*(holderY + 1))
					{
						gridPositionY = holderY;
						holderY = yGridSize + 1;
					}
					holderY++;
				}
			}
		}
		if(gridPositionX != -1 && gridPositionY != -1)
            dropObject(gridPositionX, gridPositionY);

		//System.out.println("Grid X:  " + gridPositionX + " " + "Grid Y: " + gridPositionY);
		//System.out.println("Level: " + currentLevel);
	}

	public void drawLines(Graphics2D g)
	{
		g.setColor(Color.RED);
		int xGridSize;
		int yGridSize;

		yGridSize = grid.length();
		xGridSize = grid.width();

		//compute initial screen coords
		int screenX = (26  - (6 * currentLevel - 1)) + 119;
		int screenY = (26  - (6 * currentLevel - 1)) - 1;

		//screenX = (26 * (item.x+1)) - (6 * item.z) + 120;
		//screenY = (26 * (item.y+1)) - (6 * item.z);

		//g.drawLine(screenX, screenY, screenX + 3* 26+ 32, screenY);

		int xStep = 0;
		while(xStep <= xGridSize)
		{
			g.drawLine(screenX + 26*xStep , screenY, screenX + 26*xStep, screenY + 26*grid.length());
			xStep++;
		}
		int yStep = 0;
		while(yStep <= yGridSize)
		{
			g.drawLine(screenX, screenY + 26*yStep, screenX + 26*grid.width(), screenY + 26*yStep);
			yStep++;
		}
	}

	public boolean dropObject(int x, int y)
	{
		switch(currentSprite)
		{
			//Ball
			case 0: 
				dropBall(x, y);
				break;
			
			//Block
			case 1:
				dropBlock(x,y);
				break;

			//Goal
			case 2:
				dropGoal(x,y);
				break;
			
			//Moveable Block
			case 3:
				dropMoveBlock(x,y);
				break;
			
			//Cursor
			case 4:
				dropCursor(x,y);
				break;
		}
			
			//...
		return true;
	}

	public void drawNewItems()
	{
		for(int i = 0; i <5; i++)
		{
			if(!usedSprite[i])
			{
				//get the image
				//BufferedImage bufImage = getImage(imageFileNames[i], true);
				levelSprites[i].forceX(40);
				levelSprites[i].forceY( 20 + 32 * i);
					//= new Sprite(bufImage, 40, 20 + 32 * i);
				//theGroup.add(levelSprites[i]);
			}
	
		}
	}

	public boolean dropBall(int x, int y)
	{
		Item ball = new Ball(grid,x,y,currentLevel);
		grid.set(x,y,currentLevel,ball);
		
		//create full path to sprite image
		String filename = "images/" + ball.imageFile;

		//get the image
		BufferedImage bufImage = getImage(filename, true);
		//compute initial screen coords
		int screenX = (26 * (ball.x+1)) - (6 * ball.z) + 120;
		int screenY = (26 * (ball.y+1)) - (6 * ball.z);

		//create a sprite for this item
		ball.sprite = new Piece(bufImage, screenX, screenY, ball);
		return true;
	}

	public boolean dropBlock(int x, int y)
	{
		Item block = new Block(grid,x,y,currentLevel, false);
		grid.set(x,y,currentLevel,block);
		
		//create full path to sprite image
		String filename = "images/" + block.imageFile;

		//get the image
		BufferedImage bufImage = getImage(filename, true);
		//compute initial screen coords
		int screenX = (26 * (block.x+1)) - (6 * block.z) + 120;
		int screenY = (26 * (block.y+1)) - (6 * block.z);

		//create a sprite for this item
		block.sprite = new Piece(bufImage, screenX, screenY, block);
		return true;
	}

	public boolean dropMoveBlock(int x, int y)
	{
		Item moveBlock = new Block(grid,x,y,currentLevel, true);
		grid.set(x,y,currentLevel,moveBlock);
		
		//create full path to sprite image
		String filename = "images/" + moveBlock.imageFile;

		//get the image
		BufferedImage bufImage = getImage(filename, true);
		//compute initial screen coords
		int screenX = (26 * (moveBlock.x+1)) - (6 * moveBlock.z) + 120;
		int screenY = (26 * (moveBlock.y+1)) - (6 * moveBlock.z);

		//create a sprite for this item
		moveBlock.sprite = new Piece(bufImage, screenX, screenY, moveBlock);
		return true;
	}

	public boolean dropGoal(int x, int y)
	{
		Item goal = new Goal(grid,x,y,currentLevel);
		grid.set(x,y,currentLevel,goal);
		
		//create full path to sprite image
		String filename = "images/" + goal.imageFile;

		//get the image
		BufferedImage bufImage = getImage(filename, true);
		//compute initial screen coords
		int screenX = (26 * (goal.x+1)) - (6 * goal.z) + 120;
		int screenY = (26 * (goal.y+1)) - (6 * goal.z);

		//create a sprite for this item
		goal.sprite = new Piece(bufImage, screenX, screenY, goal);
		return true;
	}

	public boolean dropCursor(int x, int y)
	{
		grid.cursorX = x;
		grid.cursorY = y;
		
		return true;
	}

	private boolean fillGrid()
	{
		for(int i = 0; i < grid.width();i++)
		{
			for(int j = 0; j< grid.length(); j++)
			{
				Item block = new Block(grid,i,j,0, false);
				grid.set(i,j,0,block);
		
				//create full path to sprite image
				String filename = "images/" + block.imageFile;

				//get the image
				BufferedImage bufImage = getImage(filename, true);
				//compute initial screen coords
				int screenX = (26 * (block.x+1)) - (6 * block.z) + 120;
				int screenY = (26 * (block.y+1)) - (6 * block.z);

				//create a sprite for this item
				block.sprite = new Piece(bufImage, screenX, screenY, block);
			}
		}

		//get cursor images
		BufferedImage normalImage = getImage("images/GrayCursor.png", true);
		BufferedImage selectImage = getImage("images/RedCursor.png", true);

		//create a sprite for the cursor at first item's coords
		Item i = grid.getTop(0, 0);
		cursor = new Cursor(i.x,i.y,i.z, normalImage, selectImage);
		cursor.item = i;
		
		return true;
		
	}

	public void saveGrid()
	{
		GauntletScanner.unparseGrid(grid, levelName);
		gridSave = true;
	}

	private boolean makeFile()
	{
		//ParserCall
		//parseIt(grid, cursorX, cursorY, levelName);

		return true;
	}
	/****************************************************************************/
	/***************************** MAIN-CLASS ***********************************/
	/****************************************************************************/

	public static void main(String[] args)
	{
		GameLoader game = new GameLoader();

		// window mode (550x420) original; I was experimenting
		// with 300x300

		grid = new Grid(Integer.parseInt(args[0]),Integer.parseInt(args[1]),Integer.parseInt(args[2])); 
		grid.cursorX = 1;
		grid.cursorY = 1;

		levelName = args[3];

		game.setup(new LevelEdit(), new Dimension(550, 420), false);
		game.start();
	}

	{ distribute = true; }

	protected void notifyError(Throwable e, String environment)
	{
		new ErrorNotificationDialog(e, bsGraphics,
			"Gumball Gauntlet", "michelhoo@yahoo.com");
	}


	/****************************************************************************/
	/**************************** SPRITE CLASS **********************************/
	/****************************************************************************/

	/****************************************************************************/
	/****************************** PIECE CLASS *********************************/
	/****************************************************************************/

	class Piece extends Sprite
	{
		//private static final int  LEFT = 1,
		//                                              RIGHT = 2,  //slide direction
		//                                          UP = 3,
		//                                          DOWN = 4;

		private static final double     MOVEMENT_SPEED = 0.4,
			ACCEL_SPEED = 0.005,
			MAXIMUM_SPEED = 1;

		/**************************** PIECE PROPERTIES ******************************/

		// actual screen coordinate (0, 0), (piecewidth, 0), (piecewidth*2, 0)
		Point iPos = new Point();

		//our Item
		Item item;

		//moving direction: NORTH, SOUTH, UP, DOWN, etc...
		Direction direction;


		/****************************************************************************/
		/****************************** CONSTRUCTOR *********************************/
		/****************************************************************************/

		// @param x, y  : grid position
		// @param num   : puzzle number
		public Piece(BufferedImage image, int screenX, int screenY, Item item)
		{
			//pass to constructor the sprite and its default screen coordinates
			super(image, screenX, screenY);

			//save current screen coordinates
			iPos.setLocation(screenX, screenY);

			//save item (2-way)
			this.item = item;
			item.sprite = this;
		}

		/****************************************************************************/
		/******************** MOVE THIS PIECE TO OTHER PIECE ************************/
		/****************************************************************************/

		//
		// slideTo()
		//
		public void slideTo()
		{
			//get the direction in which we're moving
			direction = item.move();

			//move sprite
			//add code here later

		}//end slideTo()

		//
		// reset(): move piece directly to a new location
		//
		private void reset()
		{
			//setLocation(iPos.x, iPos.y); <-set to new location
			//setSpeed(0, 0); <--stop the movement
		}

		/****************************************************************************/
		/****************************** UPDATE PIECE ********************************/
		/****************************************************************************/

		public void update(long elapsedTime)
		{
			//always call constructor
			super.update(elapsedTime);

			// move this piece
			if (direction != Direction.STOP)
			{
				// set momentum speed :)
				// increase speed by ACCEL_SPEED
				// and speed is limited to MAXIMUM_SPEED
				/*if (getHorizontalSpeed() < 0)
				 {
				 addHorizontalSpeed(elapsedTime, -ACCEL_SPEED, -MAXIMUM_SPEED);
				 }
				 if (getHorizontalSpeed() > 0)
				 {
				 addHorizontalSpeed(elapsedTime, ACCEL_SPEED, MAXIMUM_SPEED);
				 }

				 if (getVerticalSpeed() < 0)
				 {
				 addVerticalSpeed(elapsedTime, -ACCEL_SPEED, -MAXIMUM_SPEED);
				 }
				 if (getVerticalSpeed() > 0)
				 {
				 addVerticalSpeed(elapsedTime, ACCEL_SPEED, MAXIMUM_SPEED);
				 }

				 if  ((direction == LEFT     &&  getX() < swap.iPos.x) ||
				 (direction  == RIGHT    &&  getX() > swap.iPos.x) ||
				 (direction  == UP       &&  getY() < swap.iPos.y) ||
				 (direction  == DOWN     &&  getY() > swap.iPos.y))
				 {
				 // swapping has been completed, inform the game
				 finishSlide();
				 }*/
			}

		}//end update()

		public void render(Graphics2D g)
		{
			try
			{
				super.render(g);
			}
			catch (Exception e)
			{
				// catch exception when rendering null image
				// null image is the last piece without image
			}

		}//end render()

	}//end class Sprite

}//end public class LevelEdit
