package gauntlet;

import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Scanner;
import java.util.regex.MatchResult;
import java.util.TreeSet;
import java.util.Comparator;

public class LevelScanner
{

	private static ArrayList<String> levelNames;
	private static Score levelScoreLists[];
	private static ArrayList<String> levelCodeLists;

	static final Comparator<Score> SCORE_ORDER = new Comparator<Score>() 
	{
		public int compare(Score s1, Score s2) 
		{
			Long l1 = new Long(s1.getScore());
			Long l2 = new Long(s2.getScore());
			return l1.compareTo(l2);
		}
	};

	public LevelScanner(String filename)
	{
		levelNames = new ArrayList<String>();
		levelCodeLists = new ArrayList<String>();

		getLevelFile(filename);
		findScores();
	}

	private void getLevelFile(String filename)
	{
		try 
		{
			FileReader fr = new FileReader(filename);
			Scanner s = new Scanner(fr);
			while(s.hasNext()) 
			{
				levelNames.add(s.next());
				levelCodeLists.add(s.next().toUpperCase());
			}
		} 
		catch(Exception e) 
		{
			System.err.println("levels.dat File Corrupted:"+e);
			System.exit(1);
		}
	}

	private void printScores()
	{
	}

	private void findScores()
	{
		//TreeSet<Score> theScores = new TreeSet<Score>(SCORE_ORDER);
		Score holdScore;
		String levelName = new String();
		String playerName = new String();
		long playerScore = -1;
		boolean badFile = false;

		levelScoreLists = new Score[levelNames.size()];

		try 
		{
			
			FileReader fr = new FileReader("levels/scores.dat");
			
			Scanner s = new Scanner(fr);
			//Need Checks here
			while(s.hasNext() && !badFile)
			{
				levelName = s.next();
				if(s.hasNext())
				{
					playerName = s.next();
					if(s.hasNextLong())
						playerScore = s.nextLong();
					else
						badFile = true;
				}
				else
					badFile = true;
				if(!badFile)
				{
					for(int i = 0; i < levelNames.size(); i++)
					{
						if(levelName.equals(levelNames.get(i)))
						{
							levelScoreLists[i] =  new Score(playerName, playerScore);
						}
					}
				}
			}

			holdScore = new Score(playerName, playerScore);
		} 
		catch(FileNotFoundException e)
		{
			for(int i = 0; i < levelNames.size(); i++)
			{
				if(levelScoreLists[i] == null)
					levelScoreLists[i] =  new Score("Anonymous", 9999);
			}
		}
		catch(Exception e) 
		{
			holdScore = new Score("Nope", -1);
			System.err.println("caught Exception, why me?:"+e);
		}		

		for(int i = 0; i < levelNames.size(); i++)
		{
			if(levelScoreLists[i] == null)
				levelScoreLists[i] =  new Score("Anonymous", 9999);
		}


	}

	public int checkCode(String codeAttempt)
	{
		int codeLevel = -1;
		String holdString;
		for(int i = 0; i < levelCodeLists.size(); i++)
		{
			holdString = levelCodeLists.get(i);
			if(holdString.length() > 8)
				holdString = holdString.substring(0,8);
			if(codeAttempt.equals(holdString))
				codeLevel = i;
		}

		return codeLevel;
	}

	public int getLevelCount()
	{
		return levelCodeLists.size();
	}

	public String getCode(int levelIndex)
	{
		if(levelIndex < levelCodeLists.size())
			return levelCodeLists.get(levelIndex);
		else
			return null;
	}

	public Score getScore(int levelIndex)
	{
		if(levelIndex < levelScoreLists.length)
			return levelScoreLists[levelIndex];
		else
			return null;

	}


	public String getStringScore(int levelIndex)
	{
		if(levelIndex < levelScoreLists.length)
			return levelScoreLists[levelIndex].getScoreString();
		else
			return null;
	}

	public long getLongScore(int levelIndex)
	{
		if(levelIndex < levelScoreLists.length)
			return levelScoreLists[levelIndex].getScore();
		else
			return -1;
	}

	public String getStringName(int levelIndex)
	{
		if(levelIndex < levelScoreLists.length)
			return levelScoreLists[levelIndex].getName();
		else
			return null;
	}

	public int getSize()
	{
		return levelScoreLists.length;
	}

	public String getLevelName(int levelIndex)
	{
		if(levelIndex < levelNames.size())
		{
			return levelNames.get(levelIndex);
		}
		else
			return null;
	}

	public String getPrettyLevelName(int levelIndex)
	{
		if(levelIndex < levelNames.size())
		{
			return levelNames.get(levelIndex).replace('_', ' ');
		}
		else
			return null;
	}

	public void setLevelName(int levelIndex, String theName)
	{
		if(levelIndex < levelNames.size())
		{
			levelNames.set(levelIndex, theName);
		}
	}

	public void saveScores()
	{
		TreeSet<Score> theScores;
		Iterator scoreIterator;	
		FileWriter writeFile;
		try{
			writeFile = new FileWriter("levels/scores.dat");
			for(int i = 0; i < levelNames.size(); i++)
			{
				//theScores = levelScoreLists.get(i);
				//scoreIterator = theScores.iterator();
				//System.out.println(levelNames.get(i));
				writeFile.write(levelNames.get(i) + " " + levelScoreLists[i].toString() + "\n");
				
					
			}
			writeFile.close();
		}
		catch(Exception e)
		{
			System.out.println(e);
		}
	}

	public void newScore(String playerName, long theScore, int levelIndex)
	{
		//TreeSet<Score> theScores;
		
		if(levelIndex < levelScoreLists.length)
		{
			levelScoreLists[levelIndex] = new Score(playerName, theScore);
			//System.out.println("Got here");
		}
	}

	public static void main(String args[])
	{
		System.err.println("Running GauntletScanner in standalone mode...");
		if (args.length != 1) 
		{
			System.err.println("usage: java LevelScanner filename.lvl");
			System.exit(1);
		}

		try 
		{
			new LevelScanner(args[0]);
		} 
		catch (Exception e) 
		{
			System.err.println("exception caught:"+e);
			e.printStackTrace();
		}
		System.err.println("all done");
	}

	class Score
	{
		private long theScore;
		private String thePlayer;

		public Score( String playerName, long score)
		{
			theScore = score;
			thePlayer = playerName;
		}

		public long getScore()
		{
			return theScore;
		}
		
		public String getScoreString()
		{
			return "" + theScore;
		}
		public String getName()
		{
			return thePlayer;
		}

		public String toString()
		{
			return thePlayer + " " + theScore;
		}


	}

}
