
package gauntlet;

// JFC
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.ArrayList;

// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.*;
import com.golden.gamedev.object.background.*;
import com.golden.gamedev.util.ImageUtil;
import com.golden.gamedev.funbox.ErrorNotificationDialog;
import com.golden.gamedev.engine.timer.*;

import gauntlet.LevelScanner;

//
// public class Puzzle (extends Game)
//
public class Puzzle extends GameObject
{
    //
    //globals
    //

    // full puzzle image
    private BufferedImage image;

    //In Game UI Class
    private InGameUI gameUI;

    // full puzzle image size
    private int width, height;

    //Total number of options in the in game menu
    private int totalOptions = 3;

    //Current selected menu option
    private int selectedIndex = 0;

    //Win and fail booleans
    private boolean win = false;
    private boolean fail = false;

    //In Game Menu Flag
    private boolean inMenu = false;

    //Timer/Score
    private Timer scoreKeeper = new Timer(1000);
    public static long currentScore = 0;
    public static boolean writeScore = true;
		public final long MAX_SCORE_ALLOWED = 9999;

    //fonts
    private GameFont small;
    private GameFont font;

    //level names, scores, and level codes,
    //as well as saving the current scores,
    //adding new scores, score maintenance
    LevelScanner internalLevelInfo;

    //our globals
    ArrayList<String> files;
    static Grid grid;
    Cursor cursor;

    //scrolling background vars
    //Background background;
    ImageBackground imageBackground;
    Background background;
    final int VIEWSIZE = 480;
    int MAX_VIEW_X;
    int MAX_VIEW_Y;
    int MIN_VIEW_X;
    int MIN_VIEW_Y;
    double screenWidth, screenLength;

    int[] controller = ((Gumball) parent).controller;

    //for key buffering
    Direction bufferedKey = Direction.STOP;

 /****************************************************************************/
 /***************************** CONSTRUCTOR **********************************/
 /****************************************************************************/

    public Puzzle(GameEngine parent)
    {
        super(parent);
    }

 /****************************************************************************/
 /************************** INIT GAME RESOURCES *****************************/
 /****************************************************************************/
    public void initResources()
    {
        //create level object
        internalLevelInfo = ((Gumball) parent).theLevelInfo;

        // init font
        small = fontManager.getFont(new Font("Arial", Font.BOLD, 16));
        font = fontManager.getFont(new Font("Verdana", Font.BOLD, 16));

        //UI
        gameUI = new InGameUI(fontManager, getWidth(), getHeight(), "images/SideBar.png", parent);

        // init new level
        initLevel();
    }

 /****************************************************************************/
 /************************* LEVEL INITIALIZATION *****************************/
 /****************************************************************************/

    private void initLevel()
    {
			boolean levelLoaded = false;

			//set defaults for update and render methods
			updateState = DEFAULT;
			writeScore = true;

			//loop until we either successfully load a level or are unable to
			//load any level at all
			while (!levelLoaded)
			{
				//set default sentinel value
				levelLoaded = true;

				//try to load current level
				try
				{
					//get the grid
					grid = GauntletScanner.parseFile("levels/" + internalLevelInfo.getLevelName(((Gumball) parent).levelIndex) + ".lvl");
				}
				catch (Exception e)
				{
					levelLoaded = false;
				}

				//if failed, try the next level
				if (!levelLoaded)
				{
					//get handle to parent
					Gumball gumball = (Gumball)parent;

					//debug
					//System.out.println("Ouch! Bad level. Level is " + internalLevelInfo.getLevelName(gumball.levelIndex));

					//change corrupted level's name to "corrupted<level #>"
					internalLevelInfo.setLevelName(gumball.levelIndex, "corrupted" + (gumball.levelIndex+1));
					gumball.levelIndex++;
					gumball.levelContinue = gumball.levelIndex;

					//if all levels exceeded, exit
					if (gumball.levelIndex == internalLevelInfo.getLevelCount())
					{
						System.out.println("Error: Level files are corrupted. Please reinstall.");
						System.exit(-1);
					}
					else if (gumball.levelsCompleted < gumball.levelIndex)
						gumball.levelsCompleted = gumball.levelIndex;
				}

				//else it will try again when we loop

			}//end while (!levelLoaded)


			//reset UI variables
			gameUI.refreshGUI(); //purposely does not reset numResets to zero

			if(grid == null)
			{
				System.err.println("grid is null");
				System.exit(-1);
			}

			grid.balls = 0;
			currentScore = 0;
			win = false;
			fail = false;
			itemSelected = false;
			itemMoving = false;
			moveMade = false;

        //create a sprite for each playfield element
        //(blocks, balls, goals, buttons, etc.)
        //
        for (Item item : grid)
        {
						//track number of balls in the grid
            if(item instanceof Ball) grid.balls++;

            //create full path to sprite image
            String filename = "images/" + item.imageFile;

						//create array to hold BufferedImage(s)
						BufferedImage[] bufImages;

						//if it's a teleport and we want animation
						//if (grid.animateTeleports && 
						//	item instanceof Teleport && 
					 	//	((Teleport)item).animate)
						//
						if (item instanceof Teleport)
						{
							//get its images
							bufImages = getImages(filename, 2, 1, true);
						}
						else
						{
							//get the image
							bufImages = new BufferedImage[1];
							bufImages[0] = getImage(filename, true);
						}

						//allocate screen coords
						int screenX, screenY;

						//compute initial screen coords
						screenX = screenX(item.x, item.z) + item.offsetX;
						screenY = screenY(item.y, item.z) + item.offsetY;

						//create a sprite for this item
						item.sprite = new Piece(bufImages, screenX, screenY, item);

						//get sprite
						AnimatedSprite sprite = (AnimatedSprite)item.sprite;

						if (item instanceof Teleport && 
                            grid.animateTeleports    && 
                            ((Teleport)item).animate)
						{
                            int animDelay = ((Teleport)item).animDelay;
							sprite.setAnimate(true);
							sprite.setLoopAnim(true);
							sprite.setAnimationTimer(new Timer(animDelay));
						}
						else
						{
							sprite.setAnimate(false);
							sprite.setLoopAnim(false);
						}
        }

        //get cursor images
        BufferedImage normalImage = getImage("images/GrayCursor.png", true);
        BufferedImage selectImage = getImage("images/RedCursor.png", true);

        //create the cursor and associate with it the item on
        //top of which it starts out for the current level
        Item i = grid.getTop(grid.cursorX, grid.cursorY);

        //make sure those coordinates locate an item
				//code should actually never get here because GauntletScanner should
				//catch the error first, causing us to skip this level
        if (i == null)
        {
					System.out.println("Error! Initial cursor coordinates do not match any item. Exiting.");
					System.exit(-1);
        }

        //create the cursor and attach to it the to which it's
        //initially associated
        cursor = new Cursor(i.x, i.y, i.z, normalImage, selectImage);
        cursor.item = i;

        //
        // SCROLLING BACKGROUND
        //

        //initialize backgrounds

        //get an "imageBackground"--the typical background that is
        //a display that sits behind the default background or off
        //to the side. This is our in-game visible background.
        imageBackground = new ImageBackground(getImage("images/InGameBackdrop.png"));

        //get background to which all sprites are attached
        background = Background.getDefaultBackground();
        background.setClip(0, 0, VIEWSIZE, VIEWSIZE);

        //get grid *pixel* width and length
        screenWidth  = screenX(grid.width(), 0) + (26 * screenX(grid.width(), 0) / VIEWSIZE);
        screenLength = screenY(grid.length(), 0) + (26 * screenY(grid.length(), 0) / VIEWSIZE);
        if (screenWidth < VIEWSIZE)  screenWidth  = VIEWSIZE;
        if (screenLength < VIEWSIZE) screenLength = VIEWSIZE;

        //compute coords of top left corner of viewport
        //the viewport moves with the cursor, but
        //the cursor position with the viewport is
        //proportional to the cursor position within
        //the grid. If the cursor is 1/3 from the right
        //of the grid, it should be 1/3 from the right
        //of the viewport, and so on. So first we need
        //the viewports relative location in the grid.
        double cursorXProportion = cursor.getX()/screenWidth;
        double cursorYProportion = cursor.getY()/screenLength;

        //Calculate maximum x and y coords of the top-left corner
        //of the viewport
        //MIN_VIEW_X = 0;//screenX(0,grid.height());
        //MIN_VIEW_Y = 0;//screenY(0,grid.height());
        //MAX_VIEW_X = (int)(screenWidth - VIEWSIZE);
        //MAX_VIEW_Y = (int)(screenLength - VIEWSIZE);

        //set viewport x and y coordinates
        int targetX = (int)(cursor.getX() - (cursorXProportion * VIEWSIZE));
        int targetY = (int)(cursor.getY() - (cursorYProportion * VIEWSIZE));

        background.setAbsoluteLocation(targetX, targetY);

    }//end initLevel()

 /****************************************************************************/
 /***************************** UPDATE GAME **********************************/
 /****************************************************************************/

    //globals
    long keyDownLeft = 0;
    long keyDownRight= 0;
    long keyDownUp   = 0;
    long keyDownDown = 0;
    long KEY_DELAY = 100;
    boolean itemSelected = false;
    boolean itemMoving = false;
    boolean moveMade = false;

    //states
    static final int DEFAULT = 0;
    static final int MENU_MODE = 1;
    static final int WIN_POPUP_MODE = 2;
    static final int FAIL_POPUP_MODE = 3;
    static final int GAME_RESET = 4;
    static final int GAME_CLOSE = 5;
		static final int START_LEVEL = 6;

    //update state machine variable
    static int updateState = DEFAULT;

		//number of resets
		static int numResets = 0;
		static final int MAX_RESETS = 6;

    //
    // update()
    //
    public void update(long elapsedTime)
    {
        //switch according to state
        switch (updateState)
        {
            case DEFAULT:
                    defaultUpdate(elapsedTime);
                    break;

            case MENU_MODE:
                    gameUI.menu(elapsedTime);
                    break;

            case WIN_POPUP_MODE:
                    gameUI.winPopup(elapsedTime);
                    break;

            case FAIL_POPUP_MODE:
                    gameUI.failPopup(elapsedTime);
                    break;

            case GAME_RESET:
										gameUI.resetsExceeded(elapsedTime);
                    break;

            case GAME_CLOSE:
                    finish();
                    break;

						case START_LEVEL:
										initLevel();
										break;

            default:
                    //defaultUpdate(elapsedTime);
                    System.out.println("Code got to default case in update()!");
                    System.out.println("Code should never get here!!! Exiting.");
                    System.exit(-1);
                    break;
        }

        return;

    }//end update()


    //
    // defaultUpdate(): process regular cursor and item movement
    //
    public void defaultUpdate(long elapsedTime)
    {
				//flags for state change
				boolean stateChanged = false;
				boolean userHitReset = false;

        //no move made yet
        moveMade = false; //set to true by moveCursor()

        //check for normal movement; "normal" movement
        //is when an item is *not* selected. When
        //an item is already selected, the movement
        //keys have a different meaning.
        //
        if (!itemSelected)
        {
            // check for movement keypress
            //
            //if left arrow key pressed
            if (keyPressed(controller[0]))
            {
                moveCursor(cursor, Direction.WEST);
            }
            //else if right arrow key pressed
            else if (keyPressed(controller[1]))
            {
                moveCursor(cursor, Direction.EAST);
            }
            //else if up arrow key pressed
            else if (keyPressed(controller[2]))
            {
                moveCursor(cursor, Direction.NORTH);
            }
            //else if down arrow key pressed
            else if (keyPressed(controller[3]))
            {
                moveCursor(cursor, Direction.SOUTH);
            }
            //else check for user holding left arrow key
            else if (keyDown(controller[0]))
            {
                //reset other timers
                keyDownRight = keyDownUp = keyDownDown = 0;

                //need a delay, so if hold time has started
                keyDownLeft += elapsedTime;

                //else hold time started==require 100 ms
                if (keyDownLeft > KEY_DELAY)
                {
                    //prevent value overflow (which is unlikely but theoretically possible)
                    keyDownLeft = KEY_DELAY + 1;
                    moveCursor(cursor, Direction.WEST);
                }
            }
            //else if right arrow being held
            else if (keyDown(controller[1]))
            {
                //reset other timers
                keyDownLeft = keyDownUp = keyDownDown = 0;

                //start timer
                keyDownRight += elapsedTime;

                if (keyDownRight > KEY_DELAY)
                {
                        keyDownRight = KEY_DELAY + 1; //prevent value overflow
                        moveCursor(cursor, Direction.EAST);
                }
            }
            //else if up arrow key being held
            else if (keyDown(controller[2]))
            {
                //reset other timers
                keyDownLeft = keyDownRight = keyDownDown = 0;

                //start timer
                keyDownUp += elapsedTime;

                if (keyDownUp > KEY_DELAY)
                {
                        keyDownUp = KEY_DELAY + 1;
                        moveCursor(cursor, Direction.NORTH);
                }
            }
            //else if down arrow key held
            else if (keyDown(controller[3]))
            {
                //reset other timers
                keyDownLeft = keyDownRight = keyDownUp = 0;

                //start timer
                keyDownDown += elapsedTime;

                if (keyDownDown > KEY_DELAY)
                {
                        keyDownDown = KEY_DELAY + 1;
                        moveCursor(cursor, Direction.SOUTH);
                }
            }
            else
            {
                //else no movement key input
                keyDownLeft = keyDownRight = keyDownUp = keyDownDown = 0;
            }

        }//end if (!itemSelected)

        //else if an item is selected but not moving
        else if (itemSelected && !itemMoving)
        {
            //no normal movement possible when an item is selected
            keyDownLeft = keyDownRight = keyDownUp = keyDownDown = 0;

            //moveMade defaults to true
            moveMade = true;

            //
            // check for movement keypress
            //
            //if left arrow key pressed
            if (keyPressed(controller[0]) || bufferedKey == Direction.WEST)
            {
                    cursor.item.motion = Direction.WEST;
            }
            //else if right arrow key pressed
            else if (keyPressed(controller[1]) || bufferedKey == Direction.EAST)
            {
                    cursor.item.motion = Direction.EAST;
            }
            //else if up arrow key pressed
            else if (keyPressed(controller[2]) || bufferedKey == Direction.NORTH)
            {
                    cursor.item.motion = Direction.NORTH;
            }
            //else if down arrow key pressed
            else if (keyPressed(controller[3]) || bufferedKey == Direction.SOUTH)
            {
                    cursor.item.motion = Direction.SOUTH;
            }
            //else no move made
            else
            {
                    moveMade = false;
            }

            //reset buffered key
            bufferedKey = Direction.STOP;
        }
        else //item is selected and we're moving
        {
            //
            // check for movement keypress to buffer
            //
            //if left arrow key pressed
            if (keyPressed(controller[0]))
            {
                    bufferedKey = Direction.WEST;
            }
            //else if right arrow key pressed
            else if (keyPressed(controller[1]))
            {
                    bufferedKey = Direction.EAST;
            }
            //else if up arrow key pressed
            else if (keyPressed(controller[2]))
            {
                    bufferedKey = Direction.NORTH;
            }
            //else if down arrow key pressed
            else if (keyPressed(controller[3]))
            {
                    bufferedKey = Direction.SOUTH;
            }
        }

        //if no movement made, check for a selection action
        if (!moveMade && keyPressed(controller[4]))
        {
						//reset key buffer
						bufferedKey = Direction.STOP;

            //check for cursor teleport
            if (cursor.item instanceof Teleport)
            {
                Teleport tport = (Teleport)cursor.item;
                cursor.item = grid.getTop(tport.target.x, tport.target.y);
                cursor.x = cursor.item.x;
                cursor.y = cursor.item.y;
                cursor.z = cursor.item.z;
                cursor.updateLocation();

                //now update scrolling
                double cursorXProportion = cursor.getX()/screenWidth;
                double cursorYProportion = cursor.getY()/screenLength;
                int targetX = (int)(cursor.getX() - (cursorXProportion * VIEWSIZE));
                int targetY = (int)(cursor.getY() - (cursorYProportion * VIEWSIZE));
                background.setAbsoluteLocation(targetX, targetY);
            }
            //else if an item was already selected
						else if (itemSelected)
            {
                //deselect it
                cursor.normal();
                itemSelected = false;
                itemMoving = false;
            }
						//else item not yet selected, so select it
            else if (cursor.item != null && cursor.item.select()) //select it
            {
                cursor.select();
                itemSelected = true;
            }
        }

        //check for escape => enter game menu
        if (keyPressed(KeyEvent.VK_ESCAPE))
        {
					updateState = MENU_MODE;
        }

				//check for reset key
				if (keyPressed(controller[5]))
				{
					//increment number of resets
					numResets++;
					userHitReset = true;
				}

				//if max resets exceeded, offer chance to skip to next level
				if (numResets == MAX_RESETS) updateState = GAME_RESET;
				else if (userHitReset) updateState = START_LEVEL;

				//if we're still in DEFAULT state, continue
				if (updateState == DEFAULT)
				{
					//check for fail or win conditions
					if (grid.balls < 0) updateState = FAIL_POPUP_MODE;

					else if (grid.balls == 0) updateState = WIN_POPUP_MODE;

					else //neither fail nor win
					{
						//update the score
						if (scoreKeeper.action(elapsedTime))
						{
								if (currentScore < MAX_SCORE_ALLOWED)
									currentScore++;

								writeScore = true;
						}

						//
						// BEGIN CRITICAL PORTION
						//
						// The following code must always be run and
						// must be run here, the last part of this
						// update() method, and in this order.
						//
						//loop through items in the proper order
						for (Item item : grid)
						{
								//update it
								if (item != null)
								{
										Piece piece = (Piece)item.sprite;

										if (piece.isActive())
										{
												//if (item instanceof Ball && grid.balls < 1)
												if (item.removed)
												{
														//System.out.println("Problem: ball still exists");
												}
												else if (grid.offscreenUpdates || piece.isOnScreen())
													piece.update(elapsedTime);
										}
								}
						}

						//and update cursor
						cursor.update(elapsedTime);

						// END CRITICAL PORTION
						
					}//end else neither fail nor win
					
				}//end if (!stateChanged)
			
    }//end defaultUpdate()


        boolean Moving = false;

    //
    // moveCursor(): moves in direction specified if possible
    //
    public void moveCursor(Cursor cursor, Direction direction)
    {
            moveMade = true;
            int currX = cursor.x;
            int currY = cursor.y;
            Item item;
            boolean done = false;
            boolean XMove = false;
            boolean YMove = false;

            //switch according to direction
            switch (direction)
            {
                case EAST:
                case WEST:
                    //if we don't exceed the grid width
                    for (currX += direction.x; currX < grid.width() && currX >= 0
                                                     && !done; currX += direction.x)
                    {
                        item = grid.getTop(currX, currY);

                        //if not null (if there's a place to move on top of)
                        if (item != null)
                        {
                            //if the top item is a moving ball
                            if (cursor.item != item && /*item instanceof Ball &&*/ item.motion != Direction.STOP)
                            {
                                //we want to move the cursor to the item below it
                                //(you can't select a moving ball--that would make
                                //difficulties especially with level 10. Besides,
                                //by the time you moved the cursor there it would
                                //be gone before you could select it.)
                                item = grid.getBelow(currX, currY, item.z);
																Item above;
                                //try {above = grid.get(currX, currY, item.z+1);}
                                try {above = grid.get(item.x, item.y, item.z+1);}
																catch (Exception e){above = null;}
                                if (above != null && above instanceof Ramp) item = above;
																if (item == null) item = cursor.item;
                            }
                            cursor.x = item.x;
                            cursor.y = item.y; //new addition
                            cursor.z = item.z;
                            cursor.item = item;
                            cursor.updateLocation();
                            done = true;
                            XMove = true;
                        }
                    }
                break;

                    case NORTH:
                    case SOUTH:
                        //if we don't exceed the grid length
                        for (currY += direction.y; currY < grid.length() && currY >= 0
                                                         && !done; currY += direction.y)
                        {
                            item = grid.getTop(currX, currY);

                            //if not null (if there's a place to move to)
                            if (item != null)
                            {
                                //if the top item is a moving ball
                                if (cursor.item != item && /*item instanceof Ball && */ item.motion != Direction.STOP)
                                {
                                    //move to the object below it
                                    item = grid.getBelow(currX, currY, item.z);
																		Item above;
																		//try { above = grid.get(currX, currY, item.z+1);}
																		try { above = grid.get(item.x, item.y, item.z+1);}
																		catch (Exception e) { above = null; }
                                    if (above != null && above instanceof Ramp) item = above;
																		if (item == null) item = cursor.item;
                                }

                                cursor.x = item.x; //new addition
                                cursor.y = item.y;
                                cursor.z = item.z;
                                cursor.item = item;
                                cursor.updateLocation();
                                done = true;
                                YMove = true;
                            }
                        }
                        break;

                    default: //includes direction STOP and DOWN
                        cursor.item = grid.getTop(currX, currY);

                        //if there's an item at cursor position
                        if (cursor.item != null)
                        {
                                //place cursor right on the item
                                cursor.z = cursor.item.z;
                                cursor.updateLocation();
                        }
                        break;

            }//end switch

            //update scrolling
           	if (XMove) 
						{
                //the viewport moves with the cursor, but
                //the cursor position with the viewport is
                //proportional to the cursor position within
                //the grid. If the cursor is 1/3 from the right
                //of the grid, it should be 1/3 from the right
                //of the viewport, and so on. So first we need
                //the viewports relative location in the grid.
                double cursorXProportion = cursor.getX()/screenWidth;

                //set viewport x coordinates
								int cursorX = (int)cursor.getX();
								int cursorY = (int)cursor.getY();
                int targetX = (int)(cursorX - (cursorXProportion * VIEWSIZE));
								int targetY = (int)background.getY();
								if (targetY > (cursorY - 2)) targetY = cursorY - 2;
                background.setAbsoluteLocation(targetX, targetY);
            }
            else if (YMove)
            {
                //the viewport moves with the cursor, but
                //the cursor position with the viewport is
                //proportional to the cursor position within
                //the grid. If the cursor is 1/3 from the right
                //of the grid, it should be 1/3 from the right
                //of the viewport, and so on. So first we need
                //the viewports relative location in the grid.
                double cursorYProportion = cursor.getY()/screenLength;

                //set viewport y coordinates
								int cursorX = (int)cursor.getX();
								int cursorY = (int)cursor.getY();
								int targetX = (int)background.getX();
								if (targetX > (cursorX - 2)) targetX = cursorX - 2;
                int targetY = (int)(cursorY - (cursorYProportion * VIEWSIZE));
                background.setAbsoluteLocation(targetX, targetY);
            }

    }//end moveCursor()

 /****************************************************************************/
 /**************************** RENDER GAME ***********************************/
 /****************************************************************************/
    public void render(Graphics2D g)
    {
            //render starfield portion
            imageBackground.render(g, 0, 0, 0, 0, 480, 480);

            if (updateState != MENU_MODE)
            {
                //loop through items in the proper order
                for (Item item : grid)
                {
                    //render it
                    Piece piece = (Piece)item.sprite;

                    if (piece.isActive() && piece.isOnScreen())
                    {
                        //if (item instanceof Ball && grid.balls < 1)
                        if (item.removed)
                        {
                            //System.out.println("Problem: ball still exists");
                        }
                        else
                        {
                            piece.render(g);
                            if (cursor.item == item) cursor.render(g);
                        }
                    }
                }

                //render portion of sidebar that gets overwritten by
								//partially overlapping sprites--that is, it gets overwritten
								//by sprites that are partially in the viewport and partially not;
								//the render() method writes *all* of such sprites
                imageBackground.render(g, 480, 0, 480, 0, 33, 480);
            }

            //call the GUI's render method
            //gameUI.GUIrender(g);
            if (writeScore || updateState != DEFAULT)
						{
							//render sidebar
							imageBackground.render(g, 513, 0, 513, 0, 127, 480);

							//renderGUI
							gameUI.GUIrender(g);

							//reset update
							writeScore = false;
						}

    }//end render()


 /****************************************************************************/
 /****************************** PIECE CLASS *********************************/
 /****************************************************************************/

    class Piece extends AnimatedSprite
    {
        static final long ETA = 200; // ms per grid position
        Item item;                   // the item that this sprite belongs to
        Direction direction;         // includes STOP (no movement)
        int slope;
        int x,y,dx,dy;               // position & velocity
        long t;                      // time

 /****************************************************************************/
 /****************************** CONSTRUCTOR *********************************/
 /****************************************************************************/
        //
        // @param       x, y  : grid position
        // @param       num   : puzzle number
        //
        public Piece(BufferedImage[] images, int screenX, int screenY, Item item)
        {
            //pass to constructor the sprite and its default screen coordinates
            super(images, screenX, screenY);

            //save item (2-way)
            this.item = item;
            item.sprite = this;

            direction = Direction.STOP;
            x = screenX;
            y = screenY;
            dx = dy = slope = 0;
        }

 /****************************************************************************/
 /****************************** UPDATE PIECE ********************************/
 /****************************************************************************/

    public void update(long elapsedTime)
    {
            Moving = true;
            boolean done = false;
            int az;

            //always call constructor
            super.update(elapsedTime);

            //if this item is currently moving
            if (direction != Direction.STOP)
            {
                    //track time since last update
                    t += elapsedTime;

                    //while our item is not yet due in next position
                    if (t < ETA)
                    {
                            //animate (move sprite by increments)
                            //plain old "x" and "y" are *screen* coordinates,
                            //not logical coordinates
                            double xDelta = dx*t/ETA;
                            double yDelta = dy*t/ETA;
                            setLocation(x+(xDelta), y+(yDelta));

                            //if the cursor is attached to this item and item is selected,
                            //move it with the item by directly manipulating its sprite
                            //(without changing the cursor's x and y logical coordinates yet)
                            if (cursor.item==item)
                            {
                                    if (itemSelected)
                                    {
                                        //make cursor follow the item
                                        cursor.setLocation(getX(), getY());

                                        //check whether x movement or y movement of scroll
                                        if (xDelta != 0)
                                        {
                                            //scroll in x direction
                                            double cursorXProportion = cursor.getX()/screenWidth;
                                            double targetX = (int)(cursor.getX() - (cursorXProportion * VIEWSIZE));
                                            background.setAbsoluteLocation(targetX, background.getY());
                                        }
                                        else //scroll in y direction
                                        {
                                            double cursorYProportion = cursor.getY()/screenLength;
                                            double targetY = (int)(cursor.getY() - (cursorYProportion * VIEWSIZE));
                                            background.setAbsoluteLocation(background.getX(), targetY);
                                        }
                                    }
                                    else
                                    {
                                            //cursor needs to be reset to what's left under it
                                            //our item has just moved out from under it
                                            //cursor.item = grid.getBelow(cursor.x, cursor.y, cursor.z);
                                            //cursor.z--;
                                            //cursor.setLocation(cursor.item.getX(), cursor.item.getY());
                                            moveCursor(cursor, Direction.STOP);
                                    }
                            }

                            //done
                            done = true;
                    }
                    else //t >= ETA => item has arrived at new location
                    {
                            //reset time tracked
                            t = 0;

                            //update x, y and z logical values
                            az = item.z;
                            item.x += direction.x;
                            item.y += direction.y;
                            item.z += slope;
                            dx = dy = 0;

                            updatePosition();

                            //unregister the old grid position
                            grid.set(item.x - direction.x,
                                    item.y - direction.y, az, null);

                            // register receipt at new location
                            if (item.z > 0)
                            {
                                    Item below = grid.get(item.x, item.y, item.z-1);
                                    if(below!=null)
                                    {
                                            done = below.receive(item);
                                            if(!done) updatePosition();
                                    }
                            }
                    }
            }
            if (!done) //item is not moving or has just arrived at a new location
            {
                    //get the item for this piece and call the move
                    //method for that and assign direction to the
                    //result of that movement
                    direction = item.move();

                    // if item is moving
                    if (direction != Direction.STOP)
                    {
                            //if item didn't fall off the grid
                            if (item.inBounds())
                            {
                                    //calculate the screen coordinates of our destination
                                    az = grid.getSurface(item, direction);
                                    slope = az - item.z;
                                    dx = screenX(item.x+direction.x, az) - x;
                                    dy = screenY(item.y+direction.y, az) - y;

                                    //register the new grid position we're moving into
                                    grid.set(item.x + direction.x,
                                            item.y + direction.y, az, item);

                                    //if this item is the one the user selected (the one
                                    //the user interacted with directly), then make it
                                    //a "moving item"
                                    if (itemSelected && item==cursor.item) itemMoving = true;
                            }
                            else //the item fell off the grid
                            {
                                    //kill the item
                                    grid.set(item.x, item.y, item.z, null);

                                    //if this item is attached to the cursor
                                    if (item==cursor.item)
                                    {
                                            //it's gone so the cursor can't select it
                                            itemSelected = false;

                                            //set cursor sprite to normal (unselected)
                                            cursor.normal();
                                            cursor.item = null;

                                            //cursor needs to stop
                                            moveCursor(cursor, Direction.STOP);
                                    }

                                    //if it was a ball that fell off the grid,
                                    //update ball counter
                                    if (item instanceof Ball)
                                    {
                                        grid.balls = -1;
                                        if (!item.removed) item.removed = true;
                                    }

                                    //the sprite for our killed item is no
                                    //longer active
                                    setActive(false);
                            }

                    }//end if direction is not stop

            }//end if (!done)
            Moving = false;

    }//end update()

        // Synchronizes the position with that of the item's coordinates.
        void updatePosition()
        {
            x = screenX(item.x, item.z) + item.offsetX;
            y = screenY(item.y, item.z) + item.offsetY;
            setLocation(x,y);

            //if this item is attached to the cursor
            if (cursor.item==item)
            {
                if (itemSelected)
                {
                    // update cursor position
                    cursor.x = item.x;
                    cursor.y = item.y;
                    cursor.z = item.z;
                    cursor.updateLocation();

                    //check whether x movement or y movement of scroll
                    //if (cursor.getY() == cursor.getOldY())
                    {
                        //scroll in x direction
                        double cursorXProportion = cursor.getX()/screenWidth;
                        double targetX = (int)(cursor.getX() - (cursorXProportion * VIEWSIZE));
                        background.setAbsoluteLocation(targetX, background.getY());
                    }
                    //else //scroll in y direction
                    {
                        double cursorYProportion = cursor.getY()/screenLength;
                        double targetY = (int)(cursor.getY() - (cursorYProportion * VIEWSIZE));
                        background.setAbsoluteLocation(background.getX(), targetY);
                    }

                    itemMoving = false; //at least for now, the current item is no longer moving
                }
                else
                {
                    //cursor needs to be reset to what's left under it
                    //our item has just moved out from under it
                    //cursor.item = grid.getBelow(cursor.x, cursor.y, cursor.z);
                    //cursor.z--;
                    //cursor.setLocation(cursor.item.getX(), cursor.item.getY());
                    moveCursor(cursor, Direction.STOP);
                }
            }
        }//end updatePosition()

        //
        // render()
        //
        public void render(Graphics2D g)
        {
            try
            {
                super.render(g);
            }
            catch (Exception e)
            {
                // catch exception when rendering null image
                // null image is the last piece without image
            }

        }//end render()

    }//end class Piece

    //
    // screenX()
    //
    // returns the screen X coordinate given a logical x and z coordinates
    //
    public static int screenX(int x, int z)
    {
        return (26 * x) + (6 * (grid.height()-z));
    }

    //
    // screenY()
    //
    // returns the screen Y coordinate given a logical y and z coordinates
    //
    public static int screenY(int y, int z)
    {
        return (26 * y) + (6 * (grid.height()-z));
    }

}//end public class Puzzle
