package gauntlet.menu;

// JFC
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;

// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.*;

import gauntlet.Gumball;
import gauntlet.LevelScanner;


/**
 * Game BestTimes.
 */
public class BestTimes extends Menu {

	LevelScanner internalLevelInfo;
	private int bestTimeState;
	private int bestTimeStart;
	private int totalTimes;
	private int totalShownTimes;


	final int BEST_INIT = 0,
			BEST_NEITHER = 1,
			BEST_BACK = 2,
			BEST_MORE = 3,
			BEST_BOTH = 4;

 /****************************************************************************/
 /**************************** CONSTRUCTOR ***********************************/
 /****************************************************************************/

	public BestTimes(GameEngine parent) {
		super(parent);
		internalLevelInfo = ((Gumball) parent).theLevelInfo;
		totalTimes = internalLevelInfo.getLevelCount();
		bestTimeStart = 0;
		selectedIndex = 0;
	}


 /****************************************************************************/
 /*************************** INIT RESOURCES *********************************/
 /****************************************************************************/

	public void initResources() {
		super.initResources();
		bestTimeState = BEST_INIT;
		selectedIndex = 0;
	}


 /****************************************************************************/
 /***************************** UPDATE GAME **********************************/
 /****************************************************************************/

	public void update(long elapsedTime) {

		super.update(elapsedTime);

		switch(bestTimeState)
		{
				//Initialize
			case 0: 
				initBestTimes();
				break;
				//Neither
			case 1:
				bestTimeNeither();
				break;
				//Back
			case 2:
				bestTimeBack();
				break;
				//More
			case 3:
				bestTimeMore();
				break;
				//Both
			case 4:
				bestTimeBoth();
				break;
		}
	}

	private void initBestTimes()
	{
		boolean backSelect = false;
		boolean moreSelect = false;

		if(bestTimeStart > 0)
			backSelect = true;
		if(totalTimes > bestTimeStart + 13)
			moreSelect = true;

		if(backSelect && moreSelect)
			bestTimeState = BEST_BOTH;
		else if(backSelect)
			bestTimeState = BEST_BACK;
		else if(moreSelect)
			bestTimeState = BEST_MORE;
		else
			bestTimeState = BEST_NEITHER;

		totalShownTimes = Math.min((totalTimes + 1 - bestTimeStart - 1), 12);
	}

	private void bestTimeNeither()
	{
		TOTAL_OPTION = 1;
		if (keyPressed(KeyEvent.VK_ENTER) || keyPressed(KeyEvent.VK_ESCAPE)) 
		{
			playSound("sounds/MenuSelect.wav");
			finish();
		}
	}

	private void bestTimeBack()
	{
		TOTAL_OPTION = 2;
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
			switch(selectedIndex)
			{
				case 0:
					bestBack();
					break;
				case 1:
					playSound("sounds/MenuSelect.wav");
					finish();
					break;
			}
		}
	}

	private void bestTimeMore()
	{
		TOTAL_OPTION = 2;
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
			switch(selectedIndex)
			{
				case 0:
					bestMore();
					break;
				case 1:
					playSound("sounds/MenuSelect.wav");
					finish();
					break;
			}
		}
	}

	private void bestTimeBoth()
	{
		TOTAL_OPTION = 3;
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
			switch(selectedIndex)
			{
				case 0:
					bestMore();
					break;
				case 1:
					bestBack();
					break;
				case 2:
					playSound("sounds/MenuSelect.wav");
					finish();
					break;
			}
		}
	}

	private void bestMore()
	{
		if(totalTimes > bestTimeStart + 13)
			bestTimeStart = bestTimeStart + 13;

		bestTimeState = BEST_INIT;
		selectedIndex = 0;
	}

	private void bestBack()
	{
		if(bestTimeStart > 0)
			bestTimeStart = bestTimeStart -13;

		bestTimeState = BEST_INIT;
		selectedIndex = 0;
	}


 /****************************************************************************/
 /***************************** RENDER GAME **********************************/
 /****************************************************************************/

	public void render(Graphics2D g) {
		g.drawImage(menu, 0, 0, null);


		drawText(g, "BEST TIMES", 2, false);
		drawText(g, "-----------------------------", 4, false);
		drawLeftText(g, "LEVEL", 3, 140);
		drawLeftText(g, "PLAYER", 3, 280);
		drawRightText(g, "TIME", 3, -150);

		drawLevelScores(g);

		switch(bestTimeState)
		{
			case 0:
				break;
			case 1:
				bestNeitherRender(g);
				break;
			case 2:
				bestBackRender(g);
				break;
			case 3:
				bestMoreRender(g);
				break;
			case 4:
				bestBothRender(g);
				break;
		}
	}

	public void drawLevelScores(Graphics2D g)
	{


		for(int i = 0; i + bestTimeStart < totalTimes && i < 13; i++)
		{
			drawLeftText(g, formatString(internalLevelInfo.getPrettyLevelName(bestTimeStart + i), 120), 5 + i, 140);
			drawLeftText(g,  formatString(internalLevelInfo.getStringName(bestTimeStart + i), 120), 5 + i, 280 );
			drawRightText(g, formatString(internalLevelInfo.getStringScore(bestTimeStart + i), 60), 5 + i, -150);
		}

	}

	private void bestNeitherRender(Graphics2D g)
	{
		drawText(g, "Return to Main Menu", 20, true);
	}

	private void bestBackRender(Graphics2D g)
	{
		drawText(g, "Back...", 19, (selectedIndex == 0));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 1));
	}

	private void bestMoreRender(Graphics2D g)
	{
		drawText(g, "More...", 19, (selectedIndex == 0));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 1));
	}

	private void bestBothRender(Graphics2D g)
	{
		drawText(g, "More..", 18, (selectedIndex == 0));
		drawText(g, "Back...", 19, (selectedIndex == 1));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 2));
	}


	public void drawLeftText(Graphics2D g, String text, int line, int offset) 
	{
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.LEFT,
			offset, line*20, getWidth());
	}

	public void drawRightText(Graphics2D g, String text, int line, int offset) 
	{
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.RIGHT,
			offset, line*20, getWidth());
	}

	public String formatString(String theString, int width)
	{
		while(font.getWidth(theString) > width)
		{
			theString = theString.substring(0,theString.length() -1);
		}
		return theString;
	}

	void drawText(Graphics2D g, String text, int line, boolean selected) 
	{
		if (selected) 
		{
			// draw selected rectangle
			g.setColor(Color.YELLOW);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
			 (line*20) - 1,
			 font.getWidth(text) + 2,
			 18,
			 4,
			 4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.CENTER,
			0, line*20, getWidth());
	}


}
