package gauntlet.menu;

// JFC
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;

// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.*;
import com.golden.gamedev.util.FileUtil;

import gauntlet.Gumball;


/**
 * Change game controller.
 */
public class Control extends Menu {


 /************************** GAME PROPERTIES *********************************/

	// changing control, wait for user input
	boolean changeControl;
	boolean reusedKey;

	// which control to change
	int		controlNum;
	private int[] controller;


 /****************************************************************************/
 /**************************** CONSTRUCTOR ***********************************/
 /****************************************************************************/

	public Control(GameEngine parent) {
		super(parent);
		controller = ((Gumball) parent).controller;
	}


 /****************************************************************************/
 /*************************** INIT RESOURCES *********************************/
 /****************************************************************************/

	public void initResources() {
		super.initResources();

		TOTAL_OPTION = 6;
		selectedIndex = TOTAL_OPTION;
		reusedKey = false;
	}


 /****************************************************************************/
 /***************************** UPDATE GAME **********************************/
 /****************************************************************************/

	public void update(long elapsedTime) {
		if (!changeControl) {
			super.update(elapsedTime);

			// quit
			if (keyPressed(KeyEvent.VK_ESCAPE)) {
				saveController();
			}

			// select
			if (keyPressed(KeyEvent.VK_ENTER)) {
				switch (selectedIndex) {
					// return to main menu
					case 6:
						//playSound("sounds/MenuSelect.wav");
						saveController();
						finish();
					break;

					// change control
					default:
						//playSound("sounds/MenuSelect.wav");
						changeControl = true;
						controlNum = selectedIndex;
					break;
				}
			}

		} else {
			// read key input
			int input = bsInput.getKeyPressed();

			if (input != bsInput.NO_KEY) {
				changeControl = false;

				if (input != KeyEvent.VK_ESCAPE) {
					for(int i = 0; i < controller.length; i++)
					{
						if(input == controller[i] && i != controlNum)
						{
							changeControl = true;
							reusedKey = true;
						}
					}
					if(!changeControl)
					{
						((Gumball) parent).controller[selectedIndex] = input;
						reusedKey = false;
					}
				}
			}

		}
	}

	// save robo controller
	private void saveController() {
		try {
			int[] controller = ((Gumball) parent).controller;

			String[] lines = new String[controller.length*2];

			// save to lines
			for (int i=0;i < controller.length;i++) {
				switch (i) {
					case 0: lines[(i*2)] = "# LEFT   = " + KeyEvent.getKeyText(controller[i]); break;
					case 1: lines[(i*2)] = "# RIGHT  = " + KeyEvent.getKeyText(controller[i]); break;
					case 2: lines[(i*2)] = "# UP     = " + KeyEvent.getKeyText(controller[i]); break;
					case 3: lines[(i*2)] = "# DOWN   = " + KeyEvent.getKeyText(controller[i]); break;
					case 4: lines[(i*2)] = "# SELECT = " + KeyEvent.getKeyText(controller[i]); break;
					case 5: lines[(i*2)] = "# FIRE   = " + KeyEvent.getKeyText(controller[i]); break;
					//case 6: lines[(i*2)] = "# SHIFT  = " + KeyEvent.getKeyText(controller[i]); break;
				}

				lines[(i*2)+1] = String.valueOf(controller[i]);
			}

			// save to file
			FileUtil.fileWrite(lines, new java.io.File("input.dat"));
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("ERROR: Saving robot controller");
		}
	}


 /****************************************************************************/
 /***************************** RENDER GAME **********************************/
 /****************************************************************************/

	public void render(Graphics2D g) {
		g.drawImage(menu, 0, 0, null);

		// draw control
		drawText(g, "GAME CONTROL", 2, false);
		drawText(g, "------------", 3, false);

		int[] controller = ((Gumball) parent).controller;
		drawControl(g, "MOVE LEFT   : ", controller[0], 6, (selectedIndex == 0));
		drawControl(g, "MOVE RIGHT  : ", controller[1], 7, (selectedIndex == 1));
		drawControl(g, "MOVE UP     : ", controller[2], 8, (selectedIndex == 2));
		drawControl(g, "MOVE DOWN   : ", controller[3], 9, (selectedIndex == 3));

		drawControl(g, "SELECT      : ", controller[4], 11, (selectedIndex == 4));
		drawControl(g, "RESTART     : ", controller[5], 12, (selectedIndex == 5));
		//drawControl(g, "NOT USED    : ", controller[6], 13, (selectedIndex == 6));

		// return to main menu
		drawText(g, "Return to Main Menu", 20, (selectedIndex == TOTAL_OPTION));
	}

	// optimized
	private static final StringBuffer buffer = new StringBuffer();

	void drawControl(Graphics2D g, String text, int key, int line, boolean selected) {
		buffer.setLength(0);
		buffer.append(text);

		g.setColor(Color.RED);
		if (changeControl && selected) {
			// this text line (control) is currently being changed
			buffer.append("          ");
			smallFont.drawString(g,
				"Press New Keyboard Input", 200, (14*20));
			if(reusedKey)
				smallFont.drawString(g,	"Must be unique key", 200, (15*20));
		} else {
			buffer.append(KeyEvent.getKeyText(key));
			while (buffer.length() < text.length() + 10) {
				buffer.append(" ");
			}
		}

		drawText(g, buffer.toString(), line, selected);
	}

	void drawText(Graphics2D g, String text, int line, boolean selected) 
	{
		if (selected) 
		{
			// draw selected rectangle
			g.setColor(Color.YELLOW);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
			 (line*20) - 1,
			 font.getWidth(text) + 2,
			 18,
			 4,
			 4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.CENTER,
			0, line*20, getWidth());
	}


}
