package gauntlet.menu;

// JFC
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;

	// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.*;


/**
 * Game Help.
 */

public class Help extends Menu 
{


	/****************************************************************************/
	/**************************** CONSTRUCTOR ***********************************/
	/****************************************************************************/

	private Sprite cursorSprite;
	private String[] cursorStrings;
	private Sprite blockSprite;

	//Screen 1
	private Sprite ballSprite;
	private String[] ballStrings;
	private Sprite moveBlockSprite;
	private String[] moveBlockStrings;
	private Sprite goalSprite;
	private String[] goalStrings;

	//Screen 2
	private Sprite turboSprite;
	private String[] turboStrings;
	private Sprite iceSprite;
	private String[] iceStrings;
	private Sprite teleSprite;
	private String[] teleStrings;

	//Screen 3
	private Sprite pipeSprite;
	private String[] pipeStrings;
	private Sprite curvedPipeSprite;
	private String[] curvedPipeStrings;
	private Sprite voidSprite;
	private String[] voidStrings;
	

	final int HELP_INIT = 0,
			HELP_SCREEN_1 = 1,
			HELP_SCREEN_2 = 2,
			HELP_SCREEN_3 = 3;

	private int helpState;

	public Help(GameEngine parent) 
	{
		super(parent);

		
		helpState = HELP_INIT;

	}


	/****************************************************************************/
	/*************************** INIT RESOURCES *********************************/
	/****************************************************************************/

	public void initResources() 
	{
		super.initResources();
		initHelp();
		TOTAL_OPTION = 4;
	}


	/****************************************************************************/
	/***************************** UPDATE GAME **********************************/
	/****************************************************************************/

	public void update(long elapsedTime) 
	{
		super.update(elapsedTime);
		if(keyPressed(KeyEvent.VK_ENTER))
		{
			switch(selectedIndex)
			{
				case 0:
					helpState = HELP_INIT;
					break;
				case 1:
					helpState = HELP_SCREEN_1;
					break;
				case 2:
					helpState = HELP_SCREEN_2;
					break;
				case 3:
					helpState = HELP_SCREEN_3;
					break;
				case 4:
					//playSound("sounds/MenuSelect.wav");
					finish();
					break;
			}
		}
		else if (keyPressed(KeyEvent.VK_ESCAPE)) 
		{
			//playSound("sounds/MenuSelect.wav");
			finish();
		}
	}


	/****************************************************************************/
	/***************************** RENDER GAME **********************************/
	/****************************************************************************/

	public void render(Graphics2D g) 
	{
		g.drawImage(menu, 0, 0, null);

		if(helpState == HELP_INIT)
			renderInitial(g);
		else if (helpState == HELP_SCREEN_1)
			printScreen1(g);
		else if (helpState == HELP_SCREEN_2)
			printScreen2(g);
		else if (helpState == HELP_SCREEN_3)
			printScreen3(g);
	}

	private void renderInitial(Graphics2D g)
	{
		
		// draw credits
		drawText(g, "GAME HELP: Introduction", 2, false);
		drawText(g, "----------", 3, false);

		drawText(g, "Goal:", 5, false);
		drawText(g, "The goal of Gumball Gauntlet is to move all of the gumballs", 6, false);
		drawText(g, "into a hole on the game board.  When all gumballs are in a", 7, false);
		drawText(g, "hole then you win, if you lose a gumball you fail.  You may ", 8, false);
		drawText(g, "find yourself in a situation where you can no longer place ", 9, false);
		drawText(g, "gumballs into holes, in this case you must restart the game. ", 10, false);

		drawText(g, "MORE HELP:", 14, false);
		drawText(g, "Introduction", 15, (selectedIndex == 0));
		drawText(g, "Ball, Moveable Blocks, Goal", 16, (selectedIndex == 1));
		drawText(g, "Turbo Blocks, Ice, Teleporter", 17, (selectedIndex == 2));
		drawText(g, "Pipes, Curved Pipes, The Void", 18, (selectedIndex == 3));
		
		// return to main menu
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 4));
	}

	private void printScreen1(Graphics2D g)
	{
		
		// draw credits
		drawText(g, "GAME HELP: Ball, Moveable Blocks, Goal", 2, false);
		//drawText(g, "----------", 3, false);

		drawObjectHelp(g, 30, 80, ballSprite, "Ball",ballStrings,  true);
		drawObjectHelp(g, 30, 140, moveBlockSprite, "Moveable Block",moveBlockStrings, true);
		drawObjectHelp(g, 30, 200, goalSprite, "Goal", goalStrings, false);

		drawText(g, "MORE HELP:", 14, false);
		drawText(g, "Introduction", 15, (selectedIndex == 0));
		drawText(g, "Ball, Moveable Blocks, Goal", 16, (selectedIndex == 1));
		drawText(g, "Turbo Blocks, Ice, Teleporter", 17, (selectedIndex == 2));
		drawText(g, "Pipes, Curved Pipes, The Void", 18, (selectedIndex == 3));
		
		// return to main menu
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 4));
	}

	private void printScreen2(Graphics2D g)
	{
		
		// draw credits
		drawText(g, "GAME HELP: Turbo Blocks, Ice, Teleporter", 2, false);
		//drawText(g, "----------", 3, false);

		drawObjectHelp(g, 30, 80, turboSprite, "Turbo Block",turboStrings,  true);
		drawObjectHelp(g, 30, 140, iceSprite, "Ice Block",iceStrings, false);
		drawObjectHelp(g, 30, 200, teleSprite, "Teleporter", teleStrings, true);

		drawText(g, "MORE HELP:", 14, false);
		drawText(g, "Introduction", 15, (selectedIndex == 0));
		drawText(g, "Ball, Moveable Blocks, Goal", 16, (selectedIndex == 1));
		drawText(g, "Turbo Blocks, Ice, Teleporter", 17, (selectedIndex == 2));
		drawText(g, "Pipes, Curved Pipes, The Void", 18, (selectedIndex == 3));
		
		// return to main menu
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 4));
	}

	private void printScreen3(Graphics2D g)
	{
		
		// draw credits
		drawText(g, "GAME HELP: Pipes, Curved Pipes, The Void", 2, false);
		//drawText(g, "----------", 3, false);

		drawObjectHelp(g, 30, 80, pipeSprite, "Pipe",pipeStrings,  false);
		drawObjectHelp(g, 30, 140, curvedPipeSprite, "Curved Pipe",curvedPipeStrings, false);
		drawObjectHelp(g, 30, 200, voidSprite, "The Void", voidStrings, false);

		drawText(g, "MORE HELP:", 14, false);
		drawText(g, "Introduction", 15, (selectedIndex == 0));
		drawText(g, "Ball, Moveable Blocks, Goal", 16, (selectedIndex == 1));
		drawText(g, "Turbo Blocks, Ice, Teleporter", 17, (selectedIndex == 2));
		drawText(g, "Pipes, Curved Pipes, The Void", 18, (selectedIndex == 3));
		
		// return to main menu
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 4));
	}

	private void drawObjectHelp(Graphics2D g, int x, int y,  Sprite objectSprite, String objectName, String[] objectText, boolean selectable)
	{
		
		g.setColor(new Color(135,166,197));
		g.fillRoundRect(x, y, 580, 50, 5, 5);
		//objectSprite.forceX( x + 10);
		//objectSprite.forceY( y + 10);
		objectSprite.setLocation(x+10, y+10);
		objectSprite.render(g);

		g.setColor(Color.RED);
		g.drawString(objectName, x + 50, y + 20);
		if(selectable)
			g.drawString("Selectable", x+ 50, y + 40);
		g.setColor(Color.BLACK);
		g.drawString(objectText[0], x + 190, y + 20);
		g.drawString(objectText[1], x + 190, y + 40);
	}

	private void initHelp()
	{
		//Screen 1

		ballSprite = new Sprite(getImage("images/BlueBall.png"));
		ballStrings = new String[2];
		ballStrings[0] = "Balls should be moved into holes. Select a ";
		ballStrings[1] = "ball and press a direction to move the ball.";

		moveBlockSprite = new Sprite(getImage("images/YellowSquare.png"));
		moveBlockStrings = new String[2];
		moveBlockStrings[0] = "Move a selected block by pressing a ";
		moveBlockStrings[1] = "direction key. ";
		//moveBlockStrings[1] = "Select a block and press a direction to move it.";

		goalSprite = new Sprite(getImage("images/BlueHole.png"));
		goalStrings = new String[2];
		goalStrings[0] = "Goals can swallow gumballs. When all of ";
		goalStrings[1] = "the gumballs have been eaten you win.  ";

		//Screen 2

		turboSprite = new Sprite(getImage("images/TurboBlockE.png"));
		turboStrings = new String[2];
		turboStrings[0] = "Turbo blocks move balls and blocks in ";
		turboStrings[1] = "the direction of their arrows. ";

		iceSprite = new Sprite(getImage("images/IceBlock.png"));
		iceStrings = new String[2];
		iceStrings[0] = "Ice blocks cause balls and blocks to slip";
		iceStrings[1] = "causing them to slide past the ice block.";

		BufferedImage[] bufImages = getImages("images/TransporterBlack.png", 2, 1, true);
		teleSprite = new Sprite(bufImages[0]);
		teleStrings = new String[2];
		teleStrings[0] = "Teleporters move balls and blocks around.";
		teleStrings[1] = "Select a teleporter to move cursor.";

		//Screen 3

		pipeSprite = new Sprite(getImage("images/HalfPipeEW.png"));
		pipeStrings = new String[2];
		pipeStrings[0] = "Pipes allow movement of balls, but stop";
		pipeStrings[1] = "movement of blocks.";

		curvedPipeSprite = new Sprite(getImage("images/TurnNW.png"));
		curvedPipeStrings = new String[2];
		curvedPipeStrings[0] = "Curved pipes redirect the ball but stop";
		curvedPipeStrings[1] = "blocks.  ";

		voidSprite = new Sprite(getImage("images/TheVoid.png"));
		voidStrings = new String[2];
		voidStrings[0] = "If a ball falls into the void you lose.";
		voidStrings[1] = "";

		Background theBackground = voidSprite.getBackground();
		theBackground.setLocation(0,0);
		//theBackground.setSize(640,480);
	}

	void drawText(Graphics2D g, String text, int line, boolean selected) 
	{
		if (selected) 
		{
			// draw selected rectangle
			g.setColor(Color.YELLOW);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
			 (line*20) - 1,
			 font.getWidth(text) + 2,
			 18,
			 4,
			 4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.CENTER,
			0, line*20, getWidth());
	}


}
