package gauntlet.menu;

// JFC
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;

// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.*;

import gauntlet.Gumball;
import gauntlet.LevelScanner;

import java.util.ArrayList;


/**
 * Game LoadLevel.
 */
public class LoadLevel extends Menu 
{

	// changing code, wait for user input
	boolean changeCode;
	String levelCode;
	boolean cursor;

	boolean initLoading;
	boolean codeLoading;
	boolean prevLoading;

	private int prevStartLevel;
	private int totalLevels;
	private int totalShownLevels;

	// 1000 ms = 1 sec
	Timer fireRate = new Timer(500);

	int levelCompleted;

	boolean levelFound;
	boolean codeAttempt;

	int codeLevel;

	LevelScanner internalLevelInfo;

	ArrayList<String> levelNames;


	/****************************************************************************/
	/**************************** CONSTRUCTOR ***********************************/
	/****************************************************************************/

	public LoadLevel(GameEngine parent) 
	{
		super(parent);
		levelCode = new String();
		changeCode = false;
		cursor = false;
		levelFound = false;
		codeAttempt = false;
		codeLevel = -1;
		initLoading = true;
		codeLoading = false;
		prevLoading = false;
		prevStartLevel = 0;
		totalShownLevels = 0;
		levelCompleted = ((Gumball) parent).levelsCompleted;
		levelNames = new ArrayList<String>();
		internalLevelInfo = ((Gumball) parent).theLevelInfo;
		//System.out.println(internalLevelInfo.getLevelName(0));
		totalLevels = levelCompleted + 1;
		for(int i = 0; i <= levelCompleted; i++)
			levelNames.add(internalLevelInfo.getPrettyLevelName(i));
	}


	/****************************************************************************/
	/*************************** INIT RESOURCES *********************************/
	/****************************************************************************/

	public void initResources() 
	{
		super.initResources();

		//CHANGE
		TOTAL_OPTION = 2;
		selectedIndex = TOTAL_OPTION;
	}


	final int INIT_LOAD = 0,
			CODE_LOAD = 1,
			PREV_LOAD = 2;

	public int loadState = PREV_LOAD;

	/****************************************************************************/
	/***************************** UPDATE GAME **********************************/
	/****************************************************************************/
	public void update(long elapsedTime) 
	{
		super.update(elapsedTime);
		//System.out.println(selectedIndex);

		switch(loadState)
		{
			case 0:
				initialLoad();
				break;
			case 1:
				codeLoad(elapsedTime);
				break;
			case 2: 
				previousLoad();
				break;
		}
	}

	public void initialLoad()
	{
		TOTAL_OPTION = 2;
		//selectedIndex = TOTAL_OPTION;
		if(keyPressed(KeyEvent.VK_ENTER))
		{
			switch(selectedIndex)
			{
				case 0: 
					loadState = CODE_LOAD;
					initLoading = false;
					codeLoading = true;
				break;
				case 1:
					loadState = PREV_LOAD;
					initLoading = false;
					prevLoading = true;
				break;
				case 2:
					parent.nextGameID = Gumball.TITLE_SCREEN;
					finish();
				break;
			}
		}
		else if (keyPressed(KeyEvent.VK_ESCAPE)) 
		{
			//System.out.println("Yup its here");
			parent.nextGameID = Gumball.TITLE_SCREEN;
			//finish();
		}
	}

	public void codeLoad(long elapsedTime)
	{
		TOTAL_OPTION = 2;
		if (!changeCode) 
		{
			//super.update(elapsedTime);
			// quit
			if (keyPressed(KeyEvent.VK_ESCAPE)) 
			{
				//System.out.println("Yup its here");
				parent.nextGameID = Gumball.TITLE_SCREEN;
			}
			// select
			if (keyPressed(KeyEvent.VK_ENTER) && !levelFound) 
			{

				if(selectedIndex == 0)
					changeCode = true;
				else if (selectedIndex == 1)
				{
					loadState = PREV_LOAD;
					selectedIndex = 0;
					codeLoading = false;
					prevLoading = true;
				}
				else
				{
					parent.nextGameID = Gumball.TITLE_SCREEN;
					finish();
				}
			}
			if (keyPressed(KeyEvent.VK_ENTER) && levelFound) 
			{
				((Gumball) parent).levelIndex = codeLevel;
				parent.nextGameID = Gumball.GAME_MODE;
				finish();

			}
		} 
		else 
		{
			if (fireRate.action(elapsedTime)) 
			{
				if(cursor)
					cursor = false;
				else 
					cursor = true;
			}
			if (keyPressed(KeyEvent.VK_ENTER)) 
			{
				changeCode = false;
				cursor = false;
				loadLevel();
			}
			else
			{
				if(keyPressed(KeyEvent.VK_BACK_SPACE))
				{
					if(levelCode.length() > 0)
						levelCode = levelCode.substring(0,levelCode.length() - 1);
				}
				else
				{
					// read key input
					int input = bsInput.getKeyPressed();
					if (input != bsInput.NO_KEY && input >= 0x30 && input <= 0x5A && input != 0x3B && input != 0x3D) 
					{
						
						if (input != KeyEvent.VK_ESCAPE) 
						{
							if(levelCode.length() < 8)
								levelCode = levelCode + KeyEvent.getKeyText(input);
						}
					}
				}
			}
		}
	}
	
	final int PREV_INIT = 0,
			PREV_NEITHER = 1,
			PREV_MORE = 2,
			PREV_BACK = 3,
			PREV_BOTH =4;

	int prevLoadState;

	public void previousLoad()
	{
		switch (prevLoadState)
		{
			case 0: 
				initializePrevious();
				break;
			case 1:
				neitherPreviousLoad();
				break;
			case 2:
				morePreviousLoad();
				break;
			case 3:
				backPreviousLoad();
				break;
			case 4:
				bothPreviousLoad();
				break;
		}

		if (keyPressed(KeyEvent.VK_ESCAPE)) 
		{
			parent.nextGameID = Gumball.TITLE_SCREEN;
		}
	}

	private void neitherPreviousLoad()
	{
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
				
			if(selectedIndex >= 0 && selectedIndex <= totalShownLevels)
			{
				((Gumball) parent).levelIndex = selectedIndex + prevStartLevel;
				((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;
				parent.nextGameID = Gumball.GAME_MODE;
				finish();
			}

			//Load by code
			if(selectedIndex == (totalShownLevels + 1))
			{
				loadState = CODE_LOAD;
				selectedIndex = 0;
			}
			//Main Menu
			if(selectedIndex == (totalShownLevels + 2))
			{
				parent.nextGameID = Gumball.TITLE_SCREEN;
				finish();
			}
		}
	}

	private void morePreviousLoad()
	{
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
				
			if(selectedIndex >= 0 && selectedIndex <= totalShownLevels)
			{
				((Gumball) parent).levelIndex = selectedIndex + prevStartLevel;
				((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;
				parent.nextGameID = Gumball.GAME_MODE;
				finish();
			}

			//Show more levels
			if(selectedIndex == totalShownLevels + 1)
			{
				prevMore();
			}
			//Load by Code
			if(selectedIndex == totalShownLevels + 2)
			{
				loadState = CODE_LOAD;
				selectedIndex = 0;
			}
			//Main menu
			if(selectedIndex == totalShownLevels + 3)
			{
				parent.nextGameID = Gumball.TITLE_SCREEN;
				finish();
			}
		}
	}

	private void backPreviousLoad()
	{
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
				
			if(selectedIndex >= 0 && selectedIndex <= totalShownLevels)
			{
				((Gumball) parent).levelIndex = selectedIndex + prevStartLevel;
				((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;
				parent.nextGameID = Gumball.GAME_MODE;
				finish();
			}

			//Show last levels
			if(selectedIndex == totalShownLevels + 1)
			{
				prevBack();
			}
			//Load by code
			if(selectedIndex == totalShownLevels + 2)
			{
				loadState = CODE_LOAD;
				selectedIndex = 0;
			}
			//Main Menu
			if(selectedIndex == totalShownLevels + 3)
			{
				parent.nextGameID = Gumball.TITLE_SCREEN;
				finish();
			}
		}
	}

	private void bothPreviousLoad()
	{
		if (keyPressed(KeyEvent.VK_ENTER)) 
		{
				
			if(selectedIndex >= 0 && selectedIndex <= totalShownLevels)
			{
				((Gumball) parent).levelIndex = selectedIndex  + prevStartLevel;
				((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;
				parent.nextGameID = Gumball.GAME_MODE;
				finish();
			}
			
			//Show more levels
			if(selectedIndex == totalShownLevels + 1)
			{
				prevMore();
			}
			//Show previous levels
			if(selectedIndex == totalShownLevels + 2)
			{
				prevBack();
			}
			//Load by code
			if(selectedIndex == totalShownLevels + 3)
			{
				loadState = CODE_LOAD;
				selectedIndex = 0;
			}
			//Main menu
			if(selectedIndex == totalShownLevels + 4)
			{
				parent.nextGameID = Gumball.TITLE_SCREEN;
				finish();
			}
		}
	}

	private void initializePrevious()
	{
		boolean backSelect = false;
		boolean moreSelect = false;

		if(prevStartLevel > 0)
			backSelect = true;
		if(totalLevels > prevStartLevel + 13)
			moreSelect = true;

		if(backSelect && moreSelect)
			prevLoadState = PREV_BOTH;
		else if(backSelect)
			prevLoadState = PREV_BACK;
		else if(moreSelect)
			prevLoadState = PREV_MORE;
		else
			prevLoadState = PREV_NEITHER;

		//System.out.println(totalLevels);
		totalShownLevels = Math.min((totalLevels + 1 - prevStartLevel - 2), 12);
		//System.out.println(totalShownLevels);

		TOTAL_OPTION = totalShownLevels;
		//System.out.println(totalShownLevels);

		
	}

 /****************************************************************************/
 /***************************** RENDER GAME **********************************/
 /****************************************************************************/

	public void render(Graphics2D g) {

		g.drawImage(menu, 0, 0, null);

		if(loadState == INIT_LOAD)
		{
			initRender(g);
		}
		else if(loadState == CODE_LOAD)
		{
			codeRender(g);
		}
		else if(loadState == PREV_LOAD)
		{
			prevRender(g);

			switch(prevLoadState)
			{
					//Starting previous load
				case 0:
					break;
					//Load with neither option
				case 1:
					prevNeitherRender(g);
					break;
					//Load with more option
				case 2:
					prevMoreRender(g);
					break;
					//Load with back option
				case 3:
					prevBackRender(g);
					break;
					//Load with both options
				case 4:
					prevBothRender(g);
					break;
			}
		}

	}

	public void renderCompletedLevels(Graphics2D g)
	{
		int i = 0;
		while(prevStartLevel + i <= totalLevels && i < 13)
		{
			//System.out.println(i);
			drawText(g, "Level " + (prevStartLevel + i+1) +": " + levelNames.get(prevStartLevel + i) , 5 + i, (selectedIndex == i ));


			if(i == 12)
				prevStartLevel = prevStartLevel + 13;
		}
	}

	private void initRender(Graphics2D g)
	{
		drawText(g, "Enter Level Code", 14, (selectedIndex == 0));
		drawText(g, "Load Completed Level", 15, (selectedIndex == 1));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 2));
	}

	private void codeRender(Graphics2D g)
	{
		if(cursor)
			drawText(g, "Level Code: " + levelCode + "|", 5, false);
		else
			drawText(g, "Level Code: " + levelCode + " ", 5, false);

		if(changeCode)
			drawText(g, "Hit enter when done", 7, false);

		// return to main menu
		drawText(g, "Edit Code", 6, (selectedIndex == 0));

		if(levelFound)
		{
			drawText(g, "Code Correct!", 8, false);
			drawText(g, "Level: " + internalLevelInfo.getLevelName(codeLevel), 9, false);
			drawText(g, "Press ENTER to load the level!" , 10, false);
		}
		else if(codeAttempt)
		{
			drawText(g, "Code Incorrect!", 8, false);
			drawText(g, "Please Try Again", 9, false);

		}
		drawText(g, "Load Previous Level", 19, (selectedIndex == 1));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == 2));;
	}

	private void prevRender(Graphics2D g)
	{
		drawText(g, "Load Previous Level", 2, false);
		drawText(g, "--------------------", 3, false);
		//totalShownLevels = 0;

		for(int i = 0; i + prevStartLevel < totalLevels && i < 13; i++)
		{
			//System.out.println(i);
			drawText(g, "Level " + (prevStartLevel + i+1) +": " + levelNames.get(prevStartLevel + i) , 4 + i, (selectedIndex == i ));

			//if(i == 12)
				//prevStartLevel = prevStartLevel + 13;
		}
	}
	private void prevBack()
	{
		if(prevStartLevel > 0)
			prevStartLevel = prevStartLevel -13;

		prevLoadState = PREV_INIT;
		selectedIndex = 0;

	}
	private void prevMore()
	{
		if(totalLevels > prevStartLevel + 13)
			prevStartLevel = prevStartLevel + 13;

		prevLoadState = PREV_INIT;
		selectedIndex = 0;

	}

	private void prevNeitherRender(Graphics2D g)
	{
		TOTAL_OPTION = totalShownLevels + 2;
		drawText(g, "Load with Code", 19, (selectedIndex == (totalShownLevels + 1)));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == (totalShownLevels +2)));
	}

	private void prevBackRender(Graphics2D g)
	{
		TOTAL_OPTION = totalShownLevels + 3;
		drawText(g, "Back...", 18, (selectedIndex == (totalShownLevels + 1)));
		drawText(g, "Load with Code", 19, (selectedIndex == (totalShownLevels + 2)));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == (totalShownLevels +3)));
	}

	private void prevMoreRender(Graphics2D g)
	{
		TOTAL_OPTION = totalShownLevels + 3;
		drawText(g, "More...", 18, (selectedIndex == (totalShownLevels + 1)));
		drawText(g, "Load with Code", 19, (selectedIndex == (totalShownLevels + 2)));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == (totalShownLevels +3)));
	}

	private void prevBothRender(Graphics2D g)
	{
		TOTAL_OPTION = totalShownLevels + 4;
		drawText(g, "More...", 17, (selectedIndex == (totalShownLevels + 1)));
		drawText(g, "Back...", 18, (selectedIndex == (totalShownLevels + 2)));
		drawText(g, "Load with Code", 19, (selectedIndex == (totalShownLevels + 3)));
		drawText(g, "Return to Main Menu", 20, (selectedIndex == (totalShownLevels + 4)));
	}

	public void loadLevel()
	{
		//Do level loading stuff, yay!
		codeLevel = internalLevelInfo.checkCode(levelCode);
		//System.out.println("" + codeLevel);
		if(codeLevel != -1)
		{
			levelFound = true;
			codeAttempt = false;
		}
		else
		{
			codeAttempt = true;
		}
		((Gumball) parent).levelIndex = codeLevel;
		if (((Gumball) parent).levelsCompleted < ((Gumball) parent).levelIndex)
			((Gumball) parent).levelsCompleted = ((Gumball) parent).levelIndex;
		((Gumball) parent).levelContinue = ((Gumball) parent).levelIndex;
	}

	void drawText(Graphics2D g, String text, int line, boolean selected) 
	{
		if (selected) 
		{
			// draw selected rectangle
			g.setColor(Color.YELLOW);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
			 (line*20) - 1,
			 font.getWidth(text) + 2,
			 18,
			 4,
			 4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.CENTER,
			0, line*20, getWidth());
	}


}
