package gauntlet.menu;

// JFC
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;

// GTGE
import com.golden.gamedev.GameEngine;
import com.golden.gamedev.GameObject;
import com.golden.gamedev.object.GameFont;

import gauntlet.*;


/**
 * Base class for menu in this game.
 */
public class Menu extends GameObject {

	BufferedImage	title;
	BufferedImage	menu;

	GameFont		font;
	GameFont		smallFont;

	int				TOTAL_OPTION;
	int				selectedIndex;


 /****************************************************************************/
 /***************************** CONSTRUCTOR **********************************/
 /****************************************************************************/

	public Menu(GameEngine parent) {
		super(parent);
		//setFPS(200);
	}

 /****************************************************************************/
 /*************************** INIT RESOURCES *********************************/
 /****************************************************************************/

	public void initResources() {
		playMusic("sounds/Menu.mid");

		title = getImage("images/Title.png", false);
		menu = getImage("images/Menu.png", false);

		font = fontManager.getFont(new Font("Comic Sans MS",Font.BOLD ,18) );
		smallFont = fontManager.getFont(new Font("Comic Sans MS",Font.BOLD ,18));

		selectedIndex = 0;
	}


 /****************************************************************************/
 /***************************** UPDATE GAME **********************************/
 /****************************************************************************/

	public void update(long elapsedTime) {
		switch (bsInput.getKeyPressed()) {
			// up
			case KeyEvent.VK_UP:
				playSound("sounds/MenuMove.wav");
				selectedIndex--;
				if (selectedIndex < 0) selectedIndex = TOTAL_OPTION;
			break;

			// down
			case KeyEvent.VK_DOWN:
				playSound("sounds/MenuMove.wav");
				selectedIndex++;
				if (selectedIndex > TOTAL_OPTION) selectedIndex = 0;
			break;

			// back to previous level
			case KeyEvent.VK_ESCAPE:
				playSound("sounds/MenuSelect.wav");
				parent.nextGameID = Gumball.GAME_EXIT;
				finish();
			break;
		}
	}


 /****************************************************************************/
 /***************************** RENDER GAME **********************************/
 /****************************************************************************/

	public void render(Graphics2D g) {
	}

	void drawText(Graphics2D g, String text, int line, boolean selected) {
		if (selected) {
			// draw selected rectangle
			g.setColor(Color.GRAY);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
					   (line*20) - 1,
					   font.getWidth(text) + 2,
					   18,
						4,
						4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.BLACK);

		font.drawString(g, text, GameFont.CENTER,
						0, line*20, getWidth());
	}

	void drawLeftText(Graphics2D g, String text, int line, boolean selected){
		if (selected) 
		{
			// draw selected rectangle
			g.setColor(Color.GRAY);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
			 (line*20) - 1,
			 font.getWidth(text) + 2,
			 18,
			 4,
			 4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.LEFT,
			0, line*20, getWidth());
	}


}
