package gauntlet.menu;

// JFC
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.KeyEvent;

// GTGE
import com.golden.gamedev.*;
import com.golden.gamedev.object.GameFont;

import gauntlet.Gumball;


/**
 * Option, on/off fps, on/off sound.
 */
public class Option extends Menu {


 /****************************************************************************/
 /**************************** CONSTRUCTOR ***********************************/
 /****************************************************************************/

	public Option(GameEngine parent) {
		super(parent);
	}


 /****************************************************************************/
 /*************************** INIT RESOURCES *********************************/
 /****************************************************************************/

	public void initResources() {
		super.initResources();

		TOTAL_OPTION = 2;
		selectedIndex = TOTAL_OPTION;
	}


 /****************************************************************************/
 /***************************** UPDATE GAME **********************************/
 /****************************************************************************/

	public void update(long elapsedTime) {
		super.update(elapsedTime);

		// select
		if (keyPressed(KeyEvent.VK_ENTER)) {
			playSound("sounds/MenuSelect.wav");

			switch (selectedIndex) {
				// change show fps
				case 0:
					((Gumball) parent).showFPS = !((Gumball) parent).showFPS;
				break;

				// sound on/off
				case 1:
					bsMusic.setActive(!bsMusic.isActive());
					bsSound.setActive(!bsSound.isActive());

					if (bsMusic.isActive()) {
						// turn sound to on
						// replay last music
						bsMusic.play(bsMusic.getLastAudioFile());
						// equal to -> playMusic(bsMusic.getLastAudioFile());
					}
				break;

				// return to main menu
				case 2:
					finish();
				break;
			}
		}
	}


 /****************************************************************************/
 /***************************** RENDER GAME **********************************/
 /****************************************************************************/

	public void render(Graphics2D g) {
		g.drawImage(menu, 0, 0, null);

		// draw option
		drawText(g, "GAME OPTIONS", 2, false);
		drawText(g, "------------", 3, false);

		String showFPS = (((Gumball) parent).showFPS) ? " ON " : " OFF";
		drawText(g, "Show FPS     :" + showFPS, 6, (selectedIndex == 0));

		//String sound = (bsMusic.isActive()) ? " ON " : " OFF";
		//drawText(g, "Sound        :" + sound, 8, (selectedIndex == 1));

		//smallFont.drawString(g,
	  //		"type \"HYPERSPEED\" to boost the Frame Per Second!",
	  //		GameFont.CENTER, 0, 230, getWidth());

		// return to main menu
		drawText(g, "Return to Main Menu", 20, (selectedIndex == TOTAL_OPTION));
	}

	void drawText(Graphics2D g, String text, int line, boolean selected) 
	{
		if (selected) 
		{
			// draw selected rectangle
			g.setColor(Color.YELLOW);
			/*g.drawRoundRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
			 (line*20) - 1,
			 font.getWidth(text) + 2,
			 18,
			 4,
			 4);*/

			g.fillRect((getWidth()/2) - (font.getWidth(text)/2) - 1,
				(line*20) + 3,
				font.getWidth(text) + 2,
				font.getHeight() - 4 );


		}
		g.setColor(Color.RED);

		font.drawString(g, text, GameFont.CENTER,
			0, line*20, getWidth());
	}


}
